import importlib

def assertImport(module):
    try:
        m = importlib.import_module(module)
        return m
    except Exception as err:
        #print(r"引入模块[{}]异常！".format(module))
        raise AssertionError(r"import [{}] failed！because:{}".format(module,str(err)))
    pass

def assertWorks(func,*args,**kwargs):
    try:
        func.__call__(*args,**kwargs)
    except Exception as err:
        raise AssertionError("function can not worked, "+str(err))
    pass

def assertEqual(arg1,arg2,msg = None):
    assertmsg = r"{} is not equal to {}".format(arg1,arg2)
    if msg:
        assertmsg += " ({})".format(msg)
    assert arg1 == arg2,assertmsg

def assertTrue(expr,msg = None):
    if not msg:
        msg = "assert"
    assertmsg = r"{} is not a true expression".format(msg)
    
    assert expr,assertmsg

def assertFalse(expr,msg = None):
    if not msg:
        msg = "assert"
    assertmsg = r"{} is not a false expression".format(msg)
    
    assert not expr,assertmsg




import sys,unittest
from TestProxy import MethodTest
sys.path.append('.')
def test_import():
    try:
        from IutyLib.commonutil.convert import str2float
    except Exception as err:
        raise AssertionError("import module error")
from IutyLib.commonutil.convert import str2float

class TestDemo(unittest.TestCase):
    def test_str2float(self):
        v = "19.365"
        vf = str2float(v)
        self.assertEqual(vf,19.36)

    def test_str2float1(self):
        v = "19.365"
        vf = str2float(v,3)
        self.assertEqual(vf,19.36)

suite = unittest.TestSuite()
suite.addTest(TestDemo("test_str2float"))
suite.addTest(TestDemo("test_str2float1"))

with open('ut_log.txt', 'a') as fp:
    runner = unittest.TextTestRunner(stream=fp, verbosity=1)
    runner.run(suite)


from threading import Thread
import time

class LoopThread(Thread):
    '''
    construct a Thread 4 cycle task,handle must a void input method
    the proxy is a basic func,so it has no log
    args 4 func and finish
    kwargs 4 notice
    '''
    def __init__(self,targetfunc,interval = 0.01,notice = None,args = ()):
        Thread.__init__(self)
        self._interval = interval
        self._onTask = targetfunc
        self._notice = notice
        self._args = args
        self._running = False
        
        pass
    
    def threadfunc(self):
        if self._onTask != None:
            interval = self._onTask.__call__(*self._args)
            if type(interval) == float:
                self._interval = interval
        pass
    
    def stop(self):
        self._running = False
        
    def run(self):
        self._running = True
        while self._running:
            rst = self.threadfunc()
            if rst:
                self._running = False
            time.sleep(self._interval)
        self.onFin()
    
    def onFin(self):
        if not self._notice == None:
            self._notice.__call__(*self._args)
        pass
    
    def onNotice(self,**kwargs):
        if not self._notice == None:
            self._notice.__call__(**kwargs)
        pass
    
class SubThread(Thread):
    def __init__(self,targetfunc,notice = None,args = ()):
        Thread.__init__(self)
        self._notice = notice
        self._args = args
        self._onTask = targetfunc
        
        #self.start()
        pass
    
    def threadfunc(self):
        if self._onTask != None:
            self._onTask.__call__(*self._args)
            
        pass
    
        
    def run(self):
        self.threadfunc()
    
    def onFin(self):
        if not self._notice == None:
            self._notice.__call__(*self._args)
    
    def onNotice(self,**kwargs):
        if not self._notice == None:
            self._notice.__call__(**kwargs)
        pass


import sys


#sargs:源参数，一般是函数传参
#targs:目标参数，一般是缺省参数
def setArgs(sargs,targs):
    if type(sargs) == list:
        for i in range(len(sargs)):
            targs[str(i)] = sargs[i]
    
    if type(sargs) == dict:
        for k in sargs:
            targs[k] = sargs[k]
    pass

#命令行参数转传参
def parseArgs(cli = None):
    if not cli:
        cli = sys.argv
    args = []
    kwargs = {}

    key = None
    for arg in cli.split(' '):
        if key:
            kwargs[key] = arg
            key = None
            
        elif arg.startswith('-'):
            key = arg.split('-')[-1]
        
        else:
            args.append(arg)
    
    return args,kwargs

if __name__ == "__main__":
    from IutyLib.coding.asserts import *
    cli = "cliname command -p1 363 --ver 1.0.254"
    print("cli:"+cli)
    print("-"*10)
    args,kwargs = parseArgs(cli)

    print("parse res:")
    print("-"*10)
    for arg in args:
        print(arg)
    for k in kwargs:
        print("{}-{}".format(k,kwargs[k]))
    assertEqual(args[1], "command")
    
    print("set defaults:")
    print("-"*10)
    dkwargs = {"d1":"init","p1":"53505"}
    setArgs(args,dkwargs)
    setArgs(kwargs,dkwargs)

    for k in dkwargs:
        print("{}-{}".format(k,dkwargs[k]))


import os
import git
from git.repo import Repo
from git.repo.fun import is_git_dir


class GitRepository(object):
    """
    git仓库管理
    """

    def __init__(self, local_path, branch='master'):
        self.local_path = local_path
        
        self.repo = Repo(self.local_path)
        

    def setRemote(self,repo_url):
        name = os.path.split(self.local_path)[-1]
        
        remote = self.repo.create_remote(name="origin", url=repo_url)
        pass
    
    def existsRemote(self,name = "origin"):
        exists = self.repo.remote(name).exists()
        return exists
    
    def init(local_path):
        
        print(git.Repo.init(path=local_path))
        repo = GitRepository(local_path)
        return repo
        pass

    def clone(local_path, repo_url, branch = 'master'):
        """
        初始化git仓库
        :param repo_url:
        :param branch:
        :return:
        """
        
        git_local_path = os.path.join(local_path, '.git')
        if not is_git_dir(git_local_path):
            Repo.clone_from(repo_url, to_path=local_path, branch=branch)
            repo = GitRepository(local_path)
            return repo
        else:
            repo = Repo(local_path)
            return repo
        pass

    def isDirty(self):
        return self.repo.is_dirty()

    def add(self,path):
        self.repo.git.add(path)
        pass

    def commit(self,msg = "regular commit"):
        if self.isDirty():
            self.repo.git.commit("-am",msg,author="iuty")
        pass

    def pull(self):
        """
        从线上拉最新代码
        :return:
        """
        self.repo.git.pull()
    
    def push(self,name="origin",branch="master"):
        if self.existsRemote():
            self.repo.git.push(name,branch)
        pass
    
    def commitAndPush(self,msg = "regular commit",name = "origin",branch = "master"):
        self.commit(msg = msg)
        self.push(name = name,branch = branch)
        pass
    
    def branches(self):
        """
        获取所有分支
        :return:
        """
        branches = self.repo.remote().refs
        return [item.remote_head for item in branches if item.remote_head not in ['HEAD', ]]

    def commits(self):
        """
        获取所有提交记录
        :return:
        """
        commit_log = self.repo.git.log('--pretty={"commit":"%h","author":"%an","summary":"%s","date":"%cd"}',
                                       max_count=50,
                                       date='format:%Y-%m-%d %H:%M')
        log_list = commit_log.split("\n")
        return [eval(item) for item in log_list]

    def tags(self):
        """
        获取所有tag
        :return:
        """
        return [tag.name for tag in self.repo.tags]

    def changeToBranch(self, branch):
        """
        切换分值
        :param branch:
        :return:
        """
        self.repo.git.checkout(branch)

    def changeToCommit(self, branch, commit):
        """
        切换commit
        :param branch:
        :param commit:
        :return:
        """
        self.change_to_branch(branch=branch)
        self.repo.git.reset('--hard', commit)

    def createBranch(branch):
        self.repo.create_head(branch)
        pass

    def changeToTag(self, tag):
        """
        切换tag
        :param tag:
        :return:
        """
        self.repo.git.checkout(tag)


if __name__ == '__main__':
    local_path = os.path.join(r"D:\PythonSolution","py.lib")
    #GitRepository.clone(local_path,r"https://github.com/Iuty/py.lib.git")
    #GitRepository.clone(local_path,r"https://github.com/Iuty/cnn.data.lightning.git")
    #repo = GitRepository.init(local_path)
    repo = GitRepository(local_path)
    
    #repo.add("IutyLib/file/gitfiles.py")
    print(repo.isDirty())
    #repo.remote("https://github.com/Iuty/cnn.data.lightning.git")
    #repo.commit()
    #print(repo.existsRemote())
    #repo.push()
    #repo.commit()
    #branch_list = repo.branches()
    print(branch_list)
    
    #repo.pull()


import os,datetime,abc,struct,time
import random
from IutyLib.encription.encription import getfilemd5,des_descrypt,des_encrypt

class Code:
    @staticmethod
    def getFormat(s):
        if type(s) == str:
            return 's'
        if type(s) == float:
            return 'f'
        if type(s) == int:
            return 'i'
        if type(s) == datetime.date:
            return 'd'
        if type(s) == datetime.datetime:
            return 't'

    @staticmethod
    def getStringStream(data):
        bs = str.encode(data)
        st = bytes([len(bs)])
        return st+bs
        
    @staticmethod
    def getFloatStream(data):
        return struct.pack('f',data)
        
    @staticmethod
    def getIntStream(data):
        return struct.pack('I',data)
        
    @staticmethod
    def getDateStream(data):
        return bytes([data.year-1900]) + bytes([data.month]) + bytes([data.day])
    
    @staticmethod
    def getDateTimeStream(data):
        return bytes([data.year-1900]) + bytes([data.month]) + bytes([data.day]) + bytes([data.hour]) + bytes([data.minute]) + bytes([data.second])
        
    
    @staticmethod
    def getStream(format,data):
        if format == 's':
            return Code.getStringStream(data)
        if format == 'f':
            return Code.getFloatStream(data)
        if format == 'i':
            return Code.getIntStream(data)
        if format == 'd':
            return Code.getDateStream(data)
        if format == 't':
            return Code.getDateTimeStream(data)

    @staticmethod
    def getStringData(file):
        lenth = int(file.read(1)[0])
        stream = file.read(lenth)
        return bytes.decode(stream)
        
    @staticmethod
    def getFloatData(file):
        stream = file.read(4)
        return struct.unpack('f',stream)[0]
        
    @staticmethod
    def getIntData(file):
        stream = file.read(4)
        return struct.unpack('I',stream)[0]
        
    @staticmethod
    def getDateData(file):
        stream = file.read(3)
        
        return datetime.date(stream[0]+1900,stream[1],stream[2])
        
    @staticmethod
    def getDateTimeData(file):
        stream = file.read(6)
        
        return datetime.datetime(int(stream[0])+1900,int(stream[1]),int(stream[2]),
        int(stream[3]),int(stream[4]),int(stream[5]))
    
    @staticmethod
    def getData(format,file):
        if format == 's':
            return Code.getStringData(file)
        if format == 'f':
            return Code.getFloatData(file)
        if format == 'i':
            return Code.getIntData(file)
        if format == 'd':
            return Code.getDateData(file)
        if format == 't':
            return Code.getDateTimeData(file)


class ByteFile:
    #for stock, data[0] must a datetime.date
    def __init__(self,title,path,pattern,root = ".//",isserial = True):
        
        self.title = title
        self.root = root
        
        for p in path:
            self.root = self.root + p + "//"
            if not os.path.exists(self.root):
                os.mkdir(self.root)
        
        self.pattern = "." + pattern
        self.isserial = isserial
        self.value = []
        #if isserial:
            #self.value = self.getData()
        
        
    def getFile(self):
        for file in os.listdir(self.root):
            if (file[0:len(self.title)] == self.title):
                if (os.path.isfile(self.root+file)) & (file.endswith(self.pattern)):
                    return file
        return None
        
    def getAllTitle(self):
        titles = []
        for file in os.listdir(self.root):
            if (os.path.isfile(self.root+file) & (file[-4:]==self.pattern)):
                titles.append(file.split('#')[0])
        return titles
    
    def creat(self,data):
        formatstr = ''
        for s in data[0]:
            formatstr += Code.getFormat(s)
        fullpath = '{0}{1}#{2}#{3}'.format(self.root,self.title,formatstr,self.pattern)
        f = open(fullpath,'ab')
        f.close()
        
    def append(self,data):
        if type(data) == type(None):
            print ('error in {0}'.format(self.title))
        path = self.root + self.getFile()
        if self.isserial:
            f = open(path,'ab')
        else:
            f = open(path,'wb')
        for dt in data:
            for d in dt:
                stream = Code.getStream(Code.getFormat(d),d)
                f.write(stream)
        f.close()
        
    def appendData(self,data):
        if len(data) == 0:
            return
        
        appends = []
        if self.getFile() == None:
            self.creat(data)
        if (not self.isserial) | (len(self.value) == 0):
            appends = data
        else:
            lastdata = self.value[-1]
            for d in data:
                if d[0] > lastdata[0]:
                    appends.append(d)
        self.append(appends)
        
    def getData(self,refesh = False):
        if not refesh:
            if len(self.value)>0:
                return self.value
        if self.getFile() == None:
            return []
        path = self.root + self.getFile()
        f = open(path,'rb')
        rtn = []
        format = self.getFile().split('#')[1]
        filelen = f.seek(0,2)
        f.seek(0)
        while (f.seek(0,1)<filelen):
            cell = []
            data = None
            for fm in format:
                data = Code.getData(fm,f)
                
                cell.append(data)
            rtn.append(tuple(cell))
        self.value = rtn
        f.close()
        return rtn

    def removeData(self,days):
        data = self.getData()

        date = datetime.date.today() - datetime.timedelta(days)
        newdata = []
        for i in range(0,len(data)):
            if data[i][0] < date:
                newdata.append(data[i])
        self.delete()
        self.appendData(newdata)

        
    def delete(self):
        if self.getFile() != None:
            os.remove(self.root + self.getFile())
    
    def removeFiles(self):
        for file in os.listdir(self.root):
            os.remove(os.path.join(self.root,file))

class StringFile:
    def __init__(self,root = './/',title = 'txt'):
        self.title = title
        self.root = root
        pass
    
    def getFile(self):
        date = datetime.date.today().strftime('%y-%m-%d')
        for file in os.listdir(self.root):
    
            if (os.path.isfile(self.root+file)) & (file[0:len(date + '.' + self.title)] == date + '.' + self.title):
                return file
        return None
    
    def creat(self):
        date = datetime.date.today().strftime('%y-%m-%d')
        fullpath = self.root + date + '.' + self.title
        f = open(fullpath,'a')
        f.close()
    
    def append(self,message):
        if self.getFile() == None:
            self.creat()
        path = self.root + self.getFile()
        f = open(path,'a')
        f.write(message+"\n")
        f.close()
        pass

class CsvFile:
    def __init__(self,name,root="./",formatter = "csv"):
        filename = "{}.{}".format(name,formatter)
        self.fullpath = os.path.join(root,filename)
        
        pass
    
    def getInputValue(value):
        val = "{}".format(value[0])
        for i in range(1,len(value)):
            val += ",{}".format(value[i])
        rtn = val + "\n"
        return rtn
    
    def getInputValues(values):
        ds = ""
        for value in values:
            ds += CsvFile.getInputValue(value)
        return ds
    
    def append(self,value):
        fs = open(self.fullpath,"a",encoding="UTF-8")
        line = CsvFile.getInputValue(value)
        fs.write(line)
        fs.close()
        pass
    
    def appends(self,values):
        fs = open(self.fullpath,"a",encoding="UTF-8")
        ds = CsvFile.getInputValues(values)
        fs.write(ds)
        fs.close()
    
    def delete(self):
        if os.path.exists(self.fullpath):
            os.remove(self.fullpath)
        pass
    
    def getData(self):
        data0 = []
        if os.path.exists(self.fullpath):
            fs = open(self.fullpath,"r",encoding="UTF-8")
            data0 = fs.readlines()
        data = []
        for d in data0:
            dr = d.replace("\n","")
            di = dr.split(',')
            data.append(di)
        return data


if __name__ == "__main__":
    csv = CsvFile("test")
    csv.append(["No.","Value","Comment"])
    csv.appends([[0,1,"c1"],[1,2,"c2"]])
    print(csv.getData())



from IutyLib.file.files import StringFile
import os,datetime,threading

class Log(StringFile):
	def __init__(self,root = './/logs//',title = 'log'):
		if not os.path.exists(root):
			p = '.'
			ps = root.split('//')
			for i in range(1,len(ps)-1):
				
				p = p + '//' + ps[i]
				if not os.path.exists(p):
					os.makedirs(p)
		StringFile.__init__(self,root, title)
		pass
	
	def getkwargs(**kwargs):
		argmsg = ''
		for arg in kwargs:
			argmsg = argmsg + '@' + str(arg) + ':' + str(kwargs[arg]) + "; "
		return argmsg

	def appendLog(self,item,message,**kwargs):
		msg = datetime.datetime.now().strftime("%H:%M:%S.%f") + "[" + item + "]:" + message
		if len(kwargs) > 0:
			msg = msg + Log.getkwargs(**kwargs)
		self.append(msg)
		
		pass


class SimpleLog:
	def __init__(self,path = './/logs//',level = 1):
		#path => log dir
		#level:all,debug,warn,other/info,error
		#        0,    1,   2,         3,    4
		self.lock = threading.Lock()
		self.log = Log(path)
		pass

	def error(self,msg,**kwargs):
		self.lock.acquire()
		self.log.appendLog('error',msg,**kwargs)
		self.lock.release()

	def info(self,msg,**kwargs):
		self.lock.acquire()
		self.log.appendLog('info',msg,**kwargs)
		self.lock.release()

	def debug(self,msg,**kwargs):
		self.lock.acquire()
		self.log.appendLog('debug',msg,**kwargs)
		self.lock.release()

	def warn(self,msg,**kwargs):
		self.lock.acquire()
		self.log.appendLog('warn',msg,**kwargs)
		self.lock.release()

	def other(self,cmd,msg,**kwargs):
		self.lock.acquire()
		self.log.appendLog(cmd,msg,**kwargs)
		self.lock.release()

class LogManage:
	def __init__(self):
		log_d = SimpleLog()
		self.__logs__ = {'Default':log_d}
		pass

	def addLog(self,name,path='.//logs//',level = 1):
		if name in self.__logs__:
			return
		log_a = SimpleLog(path=path,level=level,title = name)
		self.__logs__[name] = log_a
		pass

	def getLog(self,name = 'Default'):
		if name in self.__logs__:
			return self.__logs__[name]
		else:
			return self.__logs__['Default']

log4py = LogManage()



#from task.tasks import TaskProxy,Task
from flask import request
import requests
import json
import os
from file.info import FileInfo,DirInfo
from commonutil.config import Config
from task.tasks import TaskItem,TaskProxy
import base64

class FileServer:
    """
    client download from server
    """
    
    def getFullPath(namespace,path):
        if namespace == None:
            #root is None => path is full path
            return os.path.join("./" ,path)
        else:
            #root not None => path is rel path,find root at os.path
            realdir = Config.get("FileServer",namespace)
            
            if realdir is None:
                raise Exception("{} not in system path".format(namespace))
            return os.path.join(realdir,path)

    def getDir(namespace,path,filter = [],choise = []):
        dpath = FileServer.getFullPath(namespace,path)
        dir = DirInfo(dpath)
        return dir.getFilesName()
        '''
        
        '''

    def getFileStream(namespace,path,filename,start,slice = 128000):
        rtn = {"exists":False}
        dirpath = FileServer.getFullPath(namespace,path)
        fi = FileInfo(dirpath,filename)
        if not fi.Exists:
            return rtn
        rtn = fi.getFileStream(start,slice)
        
        return rtn

    """
    client upload to server
    """
    def setFileStream(namespace,path,filename,start,data):
        rtn = {"success":False}
        dirpath = FileServer.getFullPath(namespace,path)
        
        fi = FileInfo(dirpath,filename)
        #if not fi.Exists:
            #return rtn
        rtn = fi.setFileStream(start,data)
        return rtn
    
    def catchFileApi():
        namespace = request.json.get("namespace")
        path = request.json.get("path")
        filename = request.json.get("filename")
        start = request.json.get("start")
        data = request.json.get("data")
        #print(bytes(data,"utf-8"))
        rtn = FileServer.setFileStream(namespace,path,filename,int(start),base64.b64decode(bytes(data,"utf-8")))
        return json.dumps(rtn)
        pass
    
    def fetchFileApi():
        namespace = request.json.get("namespace")
        path = request.json.get("path")
        filename = request.json.get("filename")
        start = request.json.get("start")
        batch = request.json.get("batch")
        
        rtn = FileServer.getFileStream(namespace,path,filename,int(start))
        #print(rtn["data"])
        rtn["data"] = str(base64.b64encode(rtn["data"]),"utf-8")
        return json.dumps(rtn)
        pass
    pass

class TransFileTask(TaskItem):
    def __init__(self,namespace,local,path,fname,type):
        self._namespace = namespace
        self._local = local
        self._path = path
        self._filename = fname
        TaskItem.__init__(self,os.path.join(local,path,fname),type)
        pass
    
    @property
    def NameSpace(self):
        return self._namespace
    
    @property
    def Local(self):
        return self._local
    
    @property
    def Path(self):
        return self._path
    
    @property
    def FileName(self):
        return self._filename
    pass

class TransDirTask(TaskItem):
    def __init__(self,namespace,local,path,type):
        self._namespace = namespace
        self._local = local
        self._path = path
        TaskItem.__init__(self,os.path.join(local,path),type)
        pass
    
    @property
    def NameSpace(self):
        return self._namespace
    
    @property
    def Local(self):
        return self._local
    
    @property
    def Path(self):
        return self._path
    
    pass

class Client(TaskProxy):
    def __init__(self,url = "http://127.0.0.1:7001/download"):
        self._url = url
        self._task = None
        
        TaskProxy.__init__(self)
        
        pass
    
    def uploadFileStream(self,namespace,local,path,fname,start=0,batch = 128000):
            
        fi = FileInfo(local,fname)
        fs = fi.getFileStream(start,batch)
        fsdata = base64.b64encode(fs["data"])
        data = {'cmd':"updateFileStream",'namespace':namespace,'path':path,'local':local,'filename':fname,'start':start,'data':str(fsdata,"utf-8")}
        
        headers = {'Content-Type':'application/json'}
        response = requests.post(url=self._url,headers = headers, data = json.dumps(data))
        result = json.loads(response.text)
        result["surplus"] = fs["surplus"]
        return result

    def downloadFileStream(self,namespace,local,path,fname,start=0,batch = 128000):
        
        data = {'cmd':"downloadFileStream",'namespace':namespace,'path':path,'local':local,'filename':fname,'start':start}
        headers = {'Content-Type':'application/json'}
        response = requests.post(url=self._url,headers = headers, data = json.dumps(data))
        
        fi = FileInfo(local,fname)
        
        result = json.loads(response.text)
        
        if result["success"]:
            dt = base64.b64decode(bytes(result.pop("data"),"utf-8"))
            rst = fi.setFileStream(start,dt)
            result["size"] = rst["size"]
        return result
    
    def uploadFile(self,namespace,local,path,filename):
        self.addTask(TransFileTask(namespace,local,path,filename,"upload"))
        pass
    
    def downloadFile(self,namespace,local,path,filename):
        self.addTask(TransFileTask(namespace,local,path,filename,"download"))
        pass

    def runTask(self):
        if self._task == None:
            self._task = self.getTask()
            
        if self._task != None:
            tsk = self._task
            if tsk.Type == "upload":
                rst = self.uploadFileStream(tsk.NameSpace,tsk.Local,tsk.Path,tsk.FileName,tsk.Start)
                if rst["success"]:
                    tsk.Start = rst["size"]
                    tsk.To = rst["surplus"]
                    
                    if tsk.Process == 1:
                        self.finTask(tsk)
                        self._task = None
                else:
                    self._task = None
            elif tsk.Type == "download":
                rst = self.downloadFileStream(tsk.NameSpace,tsk.Local,tsk.Path,tsk.FileName,tsk.Start)
                if rst["success"]:
                    tsk.Start = rst["size"]
                    tsk.To = rst["surplus"]
                    
                    if tsk.Process == 1:
                        self.finTask(tsk)
                        self._task = None
                else:
                    self._task = None
                pass
                
            return
        
        pass

if __name__ == "__main__":
    uc = Client()
    #print(uc.uploadFileStream("test",r"d:/back/local","","123.txt",0))
    #print(uc.downloadFileStream("test",r"d:/back/local","","123.txt"))
    uc.downloadFile("test",r"d:/back/local","","1-160521014039.pdf")
    uc.runTask()
    while (uc._task != None):
        
        uc.runTask()
        print(100.0*uc.Process)



import threading
import datetime
class ProcessBar:
	_cur = 0
	_total = 0
	
	def __init__(self,name):
		self._name = name
		self._start = datetime.datetime.now()
		self._lock = threading.Lock()
		pass
	
	@property
	def Status(self):
		self._lock.acquire()
		rtn = {'Name':self._name,'Start':self._start,'Cur':self._cur,'Total':self._total}
		self._lock.release()
		return rtn
	
	def setCur(self,value):
		if type(value) != int:
			return
		self._lock.acquire()
		self._cur = value
		self._lock.release()
		pass
	
	def setTotal(self,value):
		if type(value) != int:
			return
		self._lock.acquire()
		self._total = value
		self._lock.release()
		pass


import threading

class MessageQueue:
	def __init__(self):
		self.container = []
		self.lock = threading.Lock()
		pass
		
	def enQueue(self,data):
		self.lock.acquire()
		self.container.append(data)
		self.lock.release()
	
	def deQueue(self):
		self.lock.acquire()
		rtn = None
		if len(self.container) > 0:
			rtn = self.container.pop(0)
		self.lock.release()
		return rtn
	
	def clear(self):
		self.lock.acquire()
		self.container.clear()
		self.lock.release()
		pass

class TaskQueue(MessageQueue):
	def __init__(self):
		MessageQueue.__init__(self)
		pass
	
	def getCount(self):
		self.lock.acquire()
		count = len(self.container)
		self.lock.release()
		return count


# =========================================================================
import tensorflow as tf


# =========================================================================
# 网络结构定义
# 输入参数：images，image batch、4D tensor、tf.float32、[batch_size, width, height, channels]
# 返回参数：logits, float、 [batch_size, n_classes]
def inference(images, batch_size):
    # 一个简单的卷积神经网络，卷积+池化层x2，全连接层x2，最后一个softmax层做分类。
    # 卷积层1
    # 64个3x3的卷积核（3通道），padding=’SAME’，表示padding后卷积的图与原图尺寸一致，激活函数relu()
    with tf.variable_scope('conv1') as scope:
        dim = images.get_shape()[-1].value
        weights = tf.Variable(tf.truncated_normal(shape=[3, 3, dim, 64], stddev=1.0, dtype=tf.float32),name='weights', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[64]),name='biases', dtype=tf.float32)

        conv = tf.nn.conv2d(images, weights, strides=[1, 1, 1, 1], padding='SAME')
        pre_activation = tf.nn.bias_add(conv, biases)
        conv1 = tf.nn.relu(pre_activation, name=scope.name)

    # 池化层1
    # 3x3最大池化，步长strides为2，池化后执行lrn()操作，局部响应归一化，对训练有利。
    with tf.variable_scope('pooling1_lrn') as scope:
        pool1 = tf.nn.max_pool(conv1, ksize=[1, 3, 3, 1], strides=[1, 2, 2, 1], padding='SAME', name='pooling1')
        norm1 = tf.nn.lrn(pool1, depth_radius=4, bias=1.0, alpha=0.001 / 9.0, beta=0.75, name='norm1')
        
        

    # 卷积层2
    # 16个3x3的卷积核（16通道），padding=’SAME’，表示padding后卷积的图与原图尺寸一致，激活函数relu()
    with tf.variable_scope('conv2') as scope:
        dim = norm1.get_shape()[-1].value
        
        weights = tf.Variable(tf.truncated_normal(shape=[3, 3, dim, 16], stddev=0.1, dtype=tf.float32),name='weights', dtype=tf.float32)
        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[16]),name='biases', dtype=tf.float32)
        conv = tf.nn.conv2d(norm1, weights, strides=[1, 1, 1, 1], padding='SAME')
        pre_activation = tf.nn.bias_add(conv, biases)
        conv2 = tf.nn.relu(pre_activation, name='conv2')

    # 池化层2
    # 3x3最大池化，步长strides为2，池化后执行lrn()操作，
    # pool2 and norm2
    with tf.variable_scope('pooling2_lrn') as scope:
        pool2 = tf.nn.max_pool(conv2, ksize=[1, 3, 3, 1], strides=[1, 2, 2, 1], padding='SAME', name='pooling2')
        norm2 = tf.nn.lrn(pool2, depth_radius=4, bias=1.0, alpha=0.001 / 9.0, beta=0.75, name='norm2')
        

    # 全连接层3
    # 128个神经元，将之前pool层的输出reshape成一行，激活函数relu()
    with tf.variable_scope('local3') as scope:
        
        reshape = tf.reshape(norm2, shape=[batch_size, -1])
        dim = reshape.get_shape()[-1].value
        
        weights = tf.Variable(tf.truncated_normal(shape=[dim, 128], stddev=0.005, dtype=tf.float32),name='weights', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[128]),name='biases', dtype=tf.float32)
        local3 = tf.nn.relu(tf.matmul(reshape, weights) + biases, name=scope.name)

    # 全连接层4
    # 128个神经元，激活函数relu()
    with tf.variable_scope('local4') as scope:
        
        reshape = tf.reshape(local3, shape=[batch_size, -1])
        
        dim = local3.get_shape()[-1].value
        weights = tf.Variable(tf.truncated_normal(shape=[dim, 128], stddev=0.005, dtype=tf.float32),name='weights', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[128]),name='biases', dtype=tf.float32)

        local4 = tf.nn.relu(tf.matmul(local3, weights) + biases, name='local4')

    # dropout层
    #    with tf.variable_scope('dropout') as scope:
    #        drop_out = tf.nn.dropout(local4, 0.8)

    # Softmax回归层
    # 将前面的FC层输出，做一个线性回归，计算出每一类的得分
    with tf.variable_scope('softmax_linear') as scope:
        dim = local4.get_shape()[-1].value
        weights = tf.Variable(tf.truncated_normal(shape=[dim, 2], stddev=0.005, dtype=tf.float32),name='softmax_linear', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[2]),name='biases', dtype=tf.float32)

        softmax_linear = tf.add(tf.matmul(local4, weights), biases, name='softmax_linear')

    return softmax_linear


# -----------------------------------------------------------------------------
# loss计算
# 传入参数：logits，网络计算输出值。labels，真实值，在这里是0或者1
# 返回参数：loss，损失值
def losses(logits, labels):
    with tf.variable_scope('loss') as scope:
        cross_entropy = tf.nn.sparse_softmax_cross_entropy_with_logits(logits=logits, labels=labels,name='xentropy_per_example')
        loss = tf.reduce_mean(cross_entropy, name='loss')
        tf.summary.scalar(scope.name + '/loss', loss)
    return loss


# --------------------------------------------------------------------------
# loss损失值优化
# 输入参数：loss。learning_rate，学习速率。
# 返回参数：train_op，训练op，这个参数要输入sess.run中让模型去训练。
def trainning(loss, learning_rate):
    with tf.name_scope('optimizer'):
        optimizer = tf.train.AdamOptimizer(learning_rate=learning_rate)
        global_step = tf.Variable(0, name='global_step', trainable=False)
        train_op = optimizer.minimize(loss, global_step=global_step)
    return train_op


# -----------------------------------------------------------------------
# 评价/准确率计算
# 输入参数：logits，网络计算值。labels，标签，也就是真实值，在这里是0或者1。
# 返回参数：accuracy，当前step的平均准确率，也就是在这些batch中多少张图片被正确分类了。
def evaluation(logits, labels):
    with tf.variable_scope('accuracy') as scope:

        correct = tf.nn.in_top_k(logits, labels, 1)
        correct = tf.cast(correct, tf.float16)
        accuracy = tf.reduce_mean(correct)
        tf.summary.scalar(scope.name + '/accuracy', accuracy)
    return accuracy



from IutyLib.stock.files import SimTradeFile,CqcxFile
import datetime

class SimuTrader:
	def __init__(self,code,methodmark,dismark):
		self._trade = []
		self._data = []
		self._method = methodmark
		self._dis = dismark
		self._file = SimTradeFile(code,"{}_{}".format(methodmark,dismark))
		self._cqcxdata = CqcxFile(code).getData()
		self._code = code
		pass
	
	@property
	def Empty(self):
		return len(self._trade) == 0
	
	@property
	def Hold(self):
		return len(self._trade) == 2

	@property
	def Fin(self):
		return len(self._trade) == 4
	
	

	def simuBuy(self,price,daily,tex = 1.0003):
		if self.Hold:
			return
		else:
			if price == daily[2]:
				return
			if price < daily[3]:
				return
			if price > daily[2]:
				price = daily[2]
			self._trade.append(daily[0])
			self._trade.append(price*tex)
		pass

	def simuSell(self,price,daily,tex = 0.9987):
		if not self.Hold:
			return
		else:
			if price == daily[3]:
				return
			if price > daily[2]:
				return
			if price < daily[3]:
				price = daily[3]
			self._trade.append(daily[0])
			self._trade.append(price*tex)
		pass

	def setParams(self,reason,params,holdlimit = 270):
		if self.Fin:
			self._trade.append(reason)
			self._trade+=params
			if (not self.delCqcx()) & (not self.delLongHold(holdlimit)):
				self._data.append(tuple(self._trade))
			self._trade = []
		pass
	
	def delLongHold(self,holdlimit):
		if len(self._trade) >= 4:
			if (self._trade[2] - self._trade[0]).days > holdlimit:
				#print(' ')
				#print(self._trade)
				return True
		return False

	def delCqcx(self):
		if len(self._trade) >= 4:
			for cqcx in self._cqcxdata:
				if (cqcx[0] >= self._trade[0]) & (cqcx[0] <= self._trade[2]):
					#print('del cqcx code:{},date:{}'.format(self._code,cqcx[0]))
					return True
		return False

	def simuFinish(self):
		if len(self._data) > 0:
			self._file.appendData(self._data)
		pass
	
	def deleteFiles(self):
		s_file = SimTradeFile(' ',"{}_{}".format(self._method,self._dis))
		s_file.removeFiles()




from IutyLib.database.dbbase import *
import pymysql as mysql
import datetime

class MySql(SqlDataBase):
    def __init__(self,host,user,password,dbname,port=3306,**kwargs):
        SqlDataBase.__init__(self,host,user,password,dbname,port,**kwargs)
        self._db = mysql
    
    def connect(self):
        dbx = self._db.connect(host=self.host, user=self.user,port=self.port,
                                password=self.password, database=self.dbname,
                                charset='utf8')
        dbx.autocommit(True)
        return dbx





if __name__ == '__main__':
    mysql = MySql(host='localhost',port=3306,user='root',password='fastcorp',database='test')
    
    class test(mysql.Model):
        ID = Column(PrimaryKey = True,AutoIncrement = True)
        F = Column(Type=float,IsIndex = True,UnionIndex = True)
        
        S = Column(Type = str,IsIndex = True,UnionIndex = True,Length = 4)
        D = Column(Type = datetime.date,NullAble = True)
        T = Column(Type = datetime.timedelta,NullAble = True)
        #CT = Column(Type = datetime.datetime,NullAble = True)
        DT = Column(Type = datetime.datetime,NullAble = True)
        E = Column(Type = str,Enum = ['e1','e2','e3'],Default = 'e2')
        def __init__(self,name = None):
            if not name is None:
                self.__class__.__name__ = name
        
        pass
    t = test()
    
    s = datetime.datetime.now()
    t.check()
    

    for i in range(300):
        db0 = t.add(value=[{'F' : 3.14*i,'S' : i,'D':datetime.date.today(),'T':datetime.datetime.now()-s,'DT':s}])
        #db0 = t.update(value=[{'F' : 3.14*i,'S' : i,'D':datetime.date.today(),'T':datetime.datetime.now()-s,'CT':s}],where='ID={}'.format(i*2))
    db0 = t.query(orderby = 'ID desc')
    
    #db0 = t.drop()
    
    
    for d in db0:
        print(d)


from IutyLib.database.mysql import MySql
from IutyLib.database.dbbase import *

from flask import session,request,redirect,url_for,send_from_directory
from flask_restful import Resource

mysql = MySql('localhost','root','fastcorp','iuwork')

class UseRight(mysql.Model):
    uid = Column(type = str,Length = 12,IsIndex = True)
    aid = Column(type = str,Length = 40,IsIndex = True)
    right = Column()
    pass
    
class UseRightApi(Resource):
    def check():
        db = UseRight()
        db.check()
        pass
    
    def getAidList():
        uid = session.get("uid")
        rtn = {'success':False}
        if uid:
            db = UseRight()
            aids = db.query(target=r'`aid`,`right`',where=r"uid = '{}'".format(uid))
            rtn['success'] = True
            rtn['useright'] = aids
            return rtn
        else:
            rtn['error'] = 'user is not log in'
            return rtn
        pass
    
    

    def post(self):
        cmd = request.form.get('cmd')
        return UseRightApi.getAidList()
    

if __name__ == '__main__':
    #print(UseRightApi.__doc__)
    pass



import tensorflow as tf

def getConv(name,ahead,channels):
    rtn = None
    dim = ahead.get_shape()[-1].value
    with tf.variable_scope(name) as scope:
        weights = tf.Variable(tf.truncated_normal(shape=[3, 3, dim, channels], stddev=1.0, dtype=tf.float32),name='weights', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[channels]),name='biases', dtype=tf.float32)

        conv = tf.nn.conv2d(ahead, weights, strides=[1, 1, 1, 1], padding='SAME')
        pre_activation = tf.nn.bias_add(conv, biases)
        conv1 = tf.nn.relu(pre_activation, name=scope.name)
        rtn = conv1
    return rtn

def getPool(name,ahead,output):
    rtn = None
    with tf.variable_scope(name) as scope:
        pool1 = tf.nn.max_pool(ahead, ksize=[1, 3, 3, 1], strides=[1, 2, 2, 1], padding='SAME', name='pooling1')
        norm1 = tf.nn.lrn(pool1, depth_radius=4, bias=1.0, alpha=0.001 / 9.0, beta=0.75, name=scope.name)
        rtn = norm1
    return rtn

def getLocal(name,ahead,output):
    rtn = None
    with tf.variable_scope(name) as scope:
        batch_size = ahead.get_shape()[0].value
        reshape = tf.reshape(ahead, shape=[batch_size, -1])
        dim = reshape.get_shape()[-1].value
        
        weights = tf.Variable(tf.truncated_normal(shape=[dim, output], stddev=0.005, dtype=tf.float32),name='weights', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[output]),name='biases', dtype=tf.float32)
        local = tf.nn.relu(tf.matmul(reshape, weights) + biases, name=scope.name)
        rtn = local
    return rtn

def getLinear(name,ahead,output):
    rtn = None
    with tf.variable_scope(name) as scope:
        dim = ahead.get_shape()[-1].value
        weights = tf.Variable(tf.truncated_normal(shape=[dim, output], stddev=0.005, dtype=tf.float32),name='softmax_linear', dtype=tf.float32)

        biases = tf.Variable(tf.constant(value=0.1, dtype=tf.float32, shape=[output]),name='biases', dtype=tf.float32)

        softmax_linear = tf.add(tf.matmul(ahead, weights), biases, name=scope.name)
        rtn = softmax_linear
    return rtn




import baostock as bs
from IutyLib.file.files import CsvFile

class BaoStock:
    loginflg = False
    def login(self):
        if not self.loginflg:
            bs.login()
            self.loginflg = True
        pass
    
    def logout(self):
        bs.logout()
        self.loginflg = False
        pass
    
    def getKLineByCode(self,code,start="2000-1-1"):
        self.login()
        cd = "sz."+code
        if code[0] == "6":
            cd = "sh."+code
        rst = bs.query_history_k_data_plus(cd,"date,open,high,low,close,amount,volume,turn",start)
        return rst
    
    def writeCsvFile(self,root,code):
        rst = self.getKLineByCode(code)
        data = []
        while (rst.error_code == "0") & rst.next():
            data.append(rst.get_row_data())
        csv = CsvFile(code,root)
        csv.delete()
        csv.appends(data)
        
        pass
    
    def getKLineLocal(self,root,code):
        csv = CsvFile(code,root)
        kline = csv.getData()
        return kline


if __name__ == "__main__":
    
    ins = BaoStock()
    #ins.writeCsvFile(r"e:/data/test","600703")
    print(len(ins.getKLineLocal(root= r"e:/data/kline",code="600703")))


import os

errmsg=[
    "copy src is not exists",
    "copy tgt parent is not exists",
]

def removeFile(path):
    if os.path.exists(path):
        os.remove(path)
    pass

def copyFile(src,tgt):
    if os.path.isfile(src):
        if os.path.exists(os.path.dirname(tgt)):
            open(tgt,'wb').write(open(src,'rb').read())
        else:
            raise IOError(errmsg[1])
    pass

def copyTree(src,tgt,instead = False):
    src = os.path.abspath(src)
    tgt = os.path.abspath(tgt)
    
    if not os.path.exists(src):
        raise IOError(errmsg[0])
    
    if not os.path.exists(os.path.dirname(tgt)):
        raise IOError(errmsg[1])
    
    if not os.path.exists(tgt):
        os.mkdir(tgt)
    
    for p,d,f in os.walk(src):
        for di in d:
            spath = os.path.join(p,di)
            tpath = spath.replace(src,tgt)
            if not os.path.exists(tpath):
                os.mkdir(tpath)
        for fi in f:
            spath = os.path.join(p,fi)
            tpath = spath.replace(src,tgt)
            if os.path.exists(tpath):
                if instead:
                    copyFile(spath,tpath)
            else:
                copyFile(spath,tpath)
    pass


import json



class ModelOption:
    def __init__(self):
        self._configignore = []
        pass
    
    def getOption(cls:type):
        if not "_modeloption" in cls.__dict__:
            cls._modeloption = ModelOption()
        return cls._modeloption
    
    def configignore(cls:type,key:str):
        
        option = ModelOption.getOption(cls)
        option._configignore.append(key)
        return cls




def configignore(key:str):
    def addin(cls):
        ModelOption.configignore(cls,key)
        return cls
    return addin



class A:
    
    def __init__(self):
        self.a = 1
        pass
    pass
    

@configignore("a")
class B:
    
    def __init__(self):
        self.a = A()
        self.b = "B"
        
        pass
    pass

b = B()

def isclass(m):
    return not ((type(m) is int) or (type(m) is str) or (type(m) is bool) or (type(m) is dict))


def getModel(m):
    rtn = m
    if isclass(m):
        rtn = m.__dict__
        ls = list(rtn.keys())
        option = ModelOption.getOption(type(m))
        
        for k in ls:
            
            if k in option._configignore:
                rtn.pop(k)
                continue
            rtn[k] = getModel(rtn[k])
    return rtn

b.a.a = 13


def setModel(source,target):
    if isclass(source):
        source = getModel(source)
    if isclass(target):
        for si in target.__dict__:
            if si in source:
                if isclass(target.__dict__[si]):
                    setModel(source[si],target.__dict__[si])
                else:
                    target.__dict__[si] = source[si]
    pass



b1 = B()
setModel(b,b1)

print(getModel(b1))





_unittest_ = []

def unittest(testfunc):
    d = {"func":testfunc,"skip":False}
    _unittest_.append(d)
    
    return testfunc


def skip(testfunc):
    for d in _unittest_:
        if testfunc == d["func"]:
            d["skip"] = True
            break


class Event:
    def __init__(self,func):
        self.name = func.__name__
        self.cls = func.__class__
        self._deles_ = []
    
    
    
    def __iadd__(self,delegate):
        self._deles_.append(delegate)
        return self
    
    def __isub__(self,delegate):
        while delegate in self._deles_:
            self._deles_.remove(delegate)
        return self
    
    def __call__(self,*args):
        for d in self._deles_:
            d(self.name,*args)
        pass

def eve(func):
    e = Event(func)
    return e


class A:
    def __init__(self):
        self.e = []
    
    @eve
    def OnMsg(sender,*args):
        pass
        

def handle(sender,*args):
    print(sender)


if __name__ == "__main__":
    a = A()
    a.OnMsg += handle
    
    
    a1 = A()
    a1.OnMsg += handle
    a.OnMsg()


from flask import reuest
import json

def getArgs(key=None,default=None):
    #form here
    if request.headers.get("Content-Type").startswith("application/x-www-form-urlencoded"):
        if key == None:
            return request.form
        else:
            arg = request.form.get(key)
            if arg == None:
                arg = default
            return arg
                
    #json here
    pass

class ArgPaser:
    def __init__(self):
        pass





def singleton(cls):
    """
    单例装饰，在类上
    """
    def getins(*args,**kwargs):
        if not getins.ins:
            getins.ins = cls(*args,**kwargs)
        return getins.ins
    getins.ins = None
    return getins

__cents__ = {}
def center(cls):
    def login(obj):
        obj.__class__._center_.append(obj)
        pass
    
    def logout(obj):
        if obj in obj.__class__._center_:
            obj.__class__._center_.remove(obj)
            print(111)
        pass 
    
    if not cls in __cents__:
        __cents__[cls] = []
        cls._center_ = __cents__[cls]
        cls.login = login
        cls.logout = logout
        cls.__del__ = logout
    
    return cls
        

@center
class A:
    def __init__(self):
        self.a = 1
        pass


if __name__ == "__main__":
    for i in range(5):
        a = A()
    
        a.login()
        a.__del__()
        
        print(__cents__[A])


from IutyLib.file.files import ByteFile
import os,datetime

stockroot = os.environ['STOCKSERVICEPATH'] + '//db//'

class DailyFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		ByteFile.__init__(self,title,['common','daily'],'day',root)
		pass

class MinuteFile(ByteFile):
	def __init__(self,title,date):
		root = stockroot
		ByteFile.__init__(self,date.strftime('%Y%m%d'),['common','minute',title],'min',root)
		pass

class HourFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		ByteFile.__init__(self,title,['common','hour'],'hor',root)
		pass

class QuarterFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		ByteFile.__init__(self,title,['common','quarter'],'qut',root)
		pass

class StackFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		ByteFile.__init__(self,title,['common','stack'],'stk',root)
		pass

class TransFile(ByteFile):
	def __init__(self,title,date):
		root = stockroot
		ByteFile.__init__(self,date.strftime('%Y%m%d'),['trans',title],'trs',root)
		pass

class CqcxDailyFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		path = ['cqcx','daily']
		ByteFile.__init__(self,title,path,'cdy',root)
		self.daily = DailyFile(title)
		self.cqcxinfo = CqcxFile(title)
	
	def getLastCqcxDate(self):
		cqcxinfo = self.cqcxinfo.getData()
		if len(cqcxinfo) == 0:
			return datetime.date.max
		return cqcxinfo[-1][0]
	
	def write(self):
		dailydata = self.daily.getData()
		if len(dailydata) == len(self.getData()):
			return
		
		cqcxinfo = self.cqcxinfo.getData()
		needrewrite = False
		appends = []
		
		for i in range(0,len(dailydata)):
			if len(cqcxinfo) != 0:
				for cqcx in cqcxinfo:
					if dailydata[i][0] == cqcx[0]:
						ratio = (dailydata[i-1][4] - cqcx[1]/10.0 + cqcx[2] * cqcx[4] / 10.0) / (1.0 + (cqcx[4] + cqcx[3]) / 10.0) / dailydata[i-1][4]
						volratio = 1.0 + (cqcx[4] + cqcx[3]) / 10.0
						needrewrite = True
						for apindex in range(len(appends)):
							appenditem = appends[apindex]
							appendEx = (appenditem[0], appenditem[1]*ratio, appenditem[2]*ratio, appenditem[3]*ratio, appenditem[4]*ratio, appenditem[5], appenditem[6] * volratio)
							appends[apindex] = appendEx
						break
			appends.append(dailydata[i])
		
		if needrewrite:
			self.delete()
			self.getData(True)

		self.appendData(appends)
		
		
class QualifyFile(ByteFile):
	def __init__(self,title,qualify,cqcx = False):
		root = stockroot
		if cqcx:
			path = ['cqcx','qualification',qualify]
			ByteFile.__init__(self,title,path,'qlf',root)
		else:
			path = ['common','qualification',qualify]
			ByteFile.__init__(self,title,path,'qlf',root)

class BlockFile(ByteFile):
	def __init__(self,title,cqcx = False):
		root = stockroot
		if cqcx:
			path = ['cqcx','block']
		else:
			path = ['common','block']
		ByteFile.__init__(self,title,path,'blk',root,isserial = False)

class VLevelFile(ByteFile):
	def __init__(self,title,cqcx = False):
		root = stockroot
		if cqcx:
			path = ['cqcx','vlevel']
		else:
			path = ['common','vlevel']
		ByteFile.__init__(self,title,path,'vlv',root,isserial = False)

class CqcxFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		path = ['cqcx','info']
		ByteFile.__init__(self,title,path,'cqx',root)

class WeeklyFile(ByteFile):
	def __init__(self,title,cqcx = False):
		root = stockroot
		self.needrewrite = False
		if cqcx:
			ByteFile.__init__(self,title,['cqcx','weekly',],'wkl',root)
			self.daily = CqcxDailyFile(title)
			if self.getLastIndex() > 0:
				if self.getData()[-1][0] < self.daily.getLastCqcxDate():
					pass
			self.needrewrite = True
		else:
			ByteFile.__init__(self,title,['common','weekly',],'wkl',root)
			self.daily = DailyFile(title)
	
	def getLastIndex(self):
		data = self.getData()
		dailydata = self.daily.getData()
		if len(dailydata) == 0:
			return 0
		if len(data) > 0:
			if data[-1][0] == dailydata[-1][0]:
				
				return -1
			for i in range(len(data),len(dailydata)):
				if data[-1][0] < dailydata[i][0]:
					return i
			return -1
		else:
			return 0
	
	def getWeeklyData(self,date):
		dailydata = self.daily.getData(False)
		d_index = -1
		for i in range(0,len(dailydata)):
			if dailydata[i][0] == date:
				d_index = i
		if d_index > -1:
			wiy = dailydata[d_index][0].strftime('%W')
			w = list(dailydata[d_index])
			for i in range(d_index-1,-1,-1):
				wiy0 = dailydata[i][0].strftime('%W')
				if wiy != wiy0:
					break
				else:
					w[1] = dailydata[i][1]
					if dailydata[i][2] > w[2]:
						w[2] = dailydata[i][2]
					if dailydata[i][3] < w[3]:
						w[3] = dailydata[i][3]
					w[5] += dailydata[i][5]
					w[6] += dailydata[i][6]
			w[6] = w[6]//10
			return tuple(w)
		return None
	
	def write(self):
		lastindex = self.getLastIndex()
		if lastindex == -1:
			return
		dailydata = self.daily.getData()
		appends = []
		
		for i in range(lastindex,len(dailydata)):
			week = self.getWeeklyData(dailydata[i][0])
			if week != None:
				appends.append(week)
				
		if self.needrewrite:
			self.delete()
			self.getData(True)
		self.appendData(appends)
		
class MonthFile(ByteFile):
	def __init__(self,title,cqcx = False):
		root = stockroot
		self.needrewrite = False
		if cqcx:
			ByteFile.__init__(self,title,['cqcx','month',],'mth',root)
			self.daily = CqcxDailyFile(title)
			if self.getLastIndex() > 0:
				if self.getData()[-1][0] < self.daily.getLastCqcxDate():
					pass
			self.needrewrite = True
		else:
			ByteFile.__init__(self,title,['common','month',],'mth',root)
			self.daily = DailyFile(title)
	
	def getLastIndex(self):
		data = self.getData()
		dailydata = self.daily.getData()
		if len(data) > 0:
			if data[-1][0] == dailydata[-1][0]:
				
				return -1
			for i in range(len(data),len(dailydata)):
				if data[-1][0] < dailydata[i][0]:
					return i
			return -1
		else:
			return 0
	
	def getMonthData(self,date):
		dailydata = self.daily.getData(False)
		d_index = -1
		for i in range(0,len(dailydata)):
			if dailydata[i][0] == date:
				d_index = i
		if d_index > -1:
			wiy = dailydata[d_index][0].strftime('%M')
			w = list(dailydata[d_index])
			for i in range(d_index-1,-1,-1):
				wiy0 = dailydata[i][0].strftime('%M')
				if wiy != wiy0:
					break
				else:
					w[1] = dailydata[i][1]
					if dailydata[i][2] > w[2]:
						w[2] = dailydata[i][2]
					if dailydata[i][3] < w[3]:
						w[3] = dailydata[i][3]
					w[5] += dailydata[i][5]
					w[6] += dailydata[i][6]
			w[6] = w[6]//300
			return tuple(w)
		return None
	
	def write(self):
		lastindex = self.getLastIndex()
		if lastindex == -1:
			return
		dailydata = self.daily.getData()
		appends = []
		
		for i in range(lastindex,len(dailydata)):
			week = self.getMonthData(dailydata[i][0])
			if week != None:
				appends.append(week)
				
		if self.needrewrite:
			self.delete()
			self.getData(True)
		self.appendData(appends)

class AccountFile(ByteFile):
	def __init__(self,title):
		root = stockroot
		ByteFile.__init__(self,title,['account'],'act',root)
		pass
	
class SimTradeFile(ByteFile):
	def __init__(self,title,method = 'default'):
		root = stockroot
		ByteFile.__init__(self,title,['sim',method],'std',root,isserial = False)


from IutyLib.database.exceptions import *
from abc import abstractmethod
import datetime

class DataBaseParam:
    host = None
    user = None
    password = None
    dbname = None
    
    def __init__(self,host,user,password,dbname,port=3306):
        self.host = host
        self.user = user
        self.port = port
        self.password = password
        self.dbname = dbname
    
    pass

class Column:
    AutoIncrement = False
    PrimaryKey = False
    IsIndex = False
    UnionIndex = False
    IsUnique = False
    NullAble = True
    Length = None
    Default = None
    Enum = []
    
    Type = int
    
    def __init__(self,**kwargs):
        if 'AutoIncrement' in kwargs:
            self.AutoIncrement = kwargs['AutoIncrement']
        if 'NullAble' in kwargs:
            self.NullAble = kwargs['NullAble']
        if 'PrimaryKey' in kwargs:
            self.PrimaryKey = kwargs['PrimaryKey']
            if self.PrimaryKey:
                self.NullAble = False
        if 'IsIndex' in kwargs:
            self.IsIndex = kwargs['IsIndex']
        if 'UnionIndex' in kwargs:
            self.UnionIndex = kwargs['UnionIndex']
        if 'IsUnique' in kwargs:
            self.IsUnique = kwargs['IsUnique']
        if 'Length' in kwargs:
            self.Length = kwargs['Length']
        if 'Default' in kwargs:
            self.Default = kwargs['Default']
        if 'Enum' in kwargs:
            self.Enum = kwargs['Enum']
        if 'Type' in kwargs:
            self.Type = kwargs['Type']
    
    def getType(self):
        if not self.Length == None:
            length = str(self.Length)
        
        if self.Type == int:
            if self.Length == None:
                length = "11"
            return "int" + "(" + length + ")"
        
        if self.Type == float:
            return "float"
        if self.Type == str:
            if self.Length == None:
                length = "255"
            t = "varchar" + "(" + length + ")"
            if len(self.Enum) > 0:
                t = "enum("
                enumstr = ""
                for e in self.Enum:
                    if len(enumstr) > 0:
                        enumstr += ','
                    enumstr += ("\'" + e + "\'")
                t += enumstr
                t += ")"
                
            
            return t
        if self.Type == datetime.date:
            return "date"
        if (self.Type == datetime.timedelta) | (self.Type == datetime.time):
            if self.Length == None:
                length = "6"
            return "time" + "(" + length + ")"
        if self.Type == datetime.datetime:
            if self.Length == None:
                length = "6"
            return "datetime" + "(" + length + ")"
        if self.Type == bytes:
            if self.Length == None:
                length = "255"
            return "varchar" + "(" + length + ")"
    
    def getSqlStr(self):
        colstr = ""
        
        colstr += (" " + self.getType())
        if self.PrimaryKey:
            colstr += " primary key"
            self.NullAble = False
        if not self.NullAble:
            colstr += " not null"
        
        if (self.Default is None) & (self.NullAble):
            colstr += " default NULL"
        
        if not self.Default is None:
            colstr += (" Default" + '\'' + self.Default + '\'')
        
        if self.AutoIncrement:
            colstr += " AUTO_INCREMENT"
        return colstr
    pass

class SqlDataBase(DataBaseParam):
    _db = None
    
    
    def __init__(self,host,user,password,dbname,port=3306,**kwargs):
        DataBaseParam.__init__(self,host,user,password,dbname,port)
        self.Model = self.getModel.__call__()
        self._dbname = dbname
        
        pass
    
    def getModel(self):
        class Model:
            _db = self
            
            def getColumns(self):
                columns = {}
                for item in self.__class__.__dict__:
                    if self.__class__.__dict__[item].__class__ == Column:
                        columns[item] = self.__class__.__dict__[item]
                return columns
            
            def getDBColumns(self):
                tablename = self.__class__.__name__
                return self._db.getColumnDefine(tablename)
            
            def checkColumn(self):
                columns = self.getColumns()
                dbcolumns = self.getDBColumns()
                
                for col in columns:
                    if col.startswith('_'):
                        continue
                    column = columns[col]
                    checkok = False
                    for dbcolumn in dbcolumns:
                        
                        if dbcolumn['column_name'] == col:
                            
                            if column.getType() != dbcolumn['COLUMN_TYPE']:
                                self._db.alterColumn(self.__class__.__name__,col,column.getSqlStr())
                                checkok = True
                                continue
                            nullable = 'YES'
                            if not column.NullAble:
                                nullable = 'NO'
                            
                            if nullable != dbcolumn['IS_NULLABLE']:
                                self._db.alterColumn(self.__class__.__name__,col,column.getSqlStr())
                                checkok = True
                                continue
                            
                            checkok = True
                            continue
                    if not checkok:
                        #print(col)
                        self._db.addColumn(self.__class__.__name__,col,column.getSqlStr())
                    
            
            def check(self,**kwargs):
                self.creat()
                self.checkColumn()
                pass
            
            def creat(self,**kwargs):
                if self._db.isTableExists(self.__class__.__name__):
                    return
                kwargs['table'] = self.__class__.__name__.lower()
                columns = self.getColumns()
                for column in columns:
                    if not 'columns' in kwargs:
                        kwargs['columns'] = {}
                    kwargs['columns'][column] = columns[column]
                data = self._db.excuteCreat.__call__(**kwargs)
                return data
            
            def query(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteQuery.__call__(**kwargs)
                return data
            
            def add(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteAdd.__call__(**kwargs)
                return data
            
            def replace(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteReplace.__call__(**kwargs)
                return data
            
            def delete(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteDelete.__call__(**kwargs)
                return data
            
            def update(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteUpdate.__call__(**kwargs)
                return data
            
            def drop(self,**kwargs):
                kwargs['table'] = self.__class__.__name__.lower()
                data = self._db.excuteDrop.__call__(**kwargs)
                return data
            
            def tables(self,**kwargs):
                data = self._db.excuteTableInfo.__call__(**kwargs)
                return data
        return Model
    
    
    
    def isTableExists(self,tablename):
        sqlstr = "select table_name  from information_schema.tables where table_schema=\'"
        
        sqlstr += self.dbname
        sqlstr += '\' and table_type=\'base table\''
        sqlstr += ' and table_name ='
        sqlstr += (" \'" + tablename + "\'")
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        if len(data) > 0:
            return True
        return False
    
    def getColumnDefine(self,tablename):
        sqlstr = "select column_name,COLUMN_TYPE,COLUMN_KEY,IS_NULLABLE from information_schema.columns where table_schema= "
        sqlstr += ("\'" + self.dbname + "\'")
        sqlstr += " and table_name = "
        sqlstr += ("\'" + tablename + "\'")
        
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def alterColumn(self,tablename,columnname,alterstr):
        sqlstr = "alter table"
        sqlstr += (' `' + tablename + '`')
        sqlstr += " Modify Column"
        sqlstr += (' `' + columnname + '`')
        sqlstr += (' ' + alterstr)
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def addColumn(self,tablename,columnname,alterstr):
        sqlstr = "alter table"
        sqlstr += (' `' + tablename + '`')
        sqlstr += " Add Column"
        sqlstr += (' `' + columnname + '`')
        sqlstr += (' ' + alterstr)
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteCreat(self,**kwargs):
        sqlstr = "CREATE TABLE "
        if not 'table' in kwargs:
            raise TableError('Creat Has No Table')
        sqlstr += ("`" + kwargs['table'] + "`")
        if not 'columns' in kwargs:
            raise TableError('Creat Has No Column')
        colstr = ""
        indexstr = ""
        unionindexstr = ""
        for col in kwargs['columns']:
            colobj = kwargs['columns'][col]
            if len(colstr) > 0:
                colstr += ','
            colstr += ("`" + col + "`")
            colstr += colobj.getSqlStr()
            if colobj.IsIndex:
                if not colobj.UnionIndex:
                    indexstr += ("," + "Index" + " " + "`" + col + "`" + " " + "(`" + col + "`)")
                else:
                    if len(unionindexstr) > 0:
                        unionindexstr += ","
                    unionindexstr += ("`" + col + "`")
        if len(unionindexstr) > 0:
            unionindexstr = "," + "Index" + " " + "`union`" + " " + "(" + unionindexstr + ")"
            
        sqlstr += ("(" + colstr + indexstr + unionindexstr +")")
        
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
        
    
    def excuteDrop(self,**kwargs):
        sqlstr = "DROP TABLE"
        if not 'table' in kwargs:
            raise TableError('Drop Has No Table')
        sqlstr += ("`" + kwargs['table'] + "`")
        
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteTableInfo(self,**kwargs):
        sqlstr = "SELECT TABLE_NAME FROM information_schema.TABLES where Table_SCHEMA = '{0}'".format(self._dbname)
        if 'orderby' in kwargs:
            sqlstr += (" " + "order by" + " " + kwargs['orderby'])
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
            
    
    @abstractmethod
    def connect(self):
        pass
    
    def excuteSql(self,sqlstr):
        dbx = self.connect()
        db0 = dbx.cursor(self._db.cursors.DictCursor)
        db0.execute(sqlstr)
        db0.close()
        dbx.close()
        return db0

    def excuteQuery(self,**kwargs):
        sqlstr = "select "
        target = '*'
        if 'target' in kwargs:
            target = kwargs['target']
        sqlstr += (target + ' ')
        if not 'table' in kwargs:
            raise QueryError('Query Has No Table')
        sqlstr += ('from ' + "`" + kwargs['table'] + "`" + ' ')
        if 'join' in kwargs:
            sqlstr += (kwargs['join'] + ' ')
        if 'where' in kwargs:
            sqlstr += ("where " + kwargs['where'] + ' ')
        if 'groupby' in kwargs:
            sqlstr += ("group by " + kwargs['groupby'] + ' ')
        if 'orderby' in kwargs:
            sqlstr += ("order by " + kwargs['orderby'] + ' ')
        #Having here
        if 'limit' in kwargs:
            sqlstr += ("limit " + kwargs['limit'] + ' ')
        
        #log here
        #print(sqlstr)
        
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteAdd(self,**kwargs):
        sqlstr = 'insert into '
        if not 'table' in kwargs:
            raise AddError('Add Has No Table')
        sqlstr += ("`" + kwargs['table'] + "`")
        if not 'value' in kwargs:
            raise AddError('Add Has No Value')
        fields = ""
        values = ""
        firstvalue = kwargs['value'].pop(0)
        for v in firstvalue:
            if len(fields) > 0:
                fields+=","
                values+=","
            fields += r"`{}`".format(v)
            if firstvalue[v] == None:
                values += 'null'
            else:
                values += '\'{0}\''.format(firstvalue[v])
        sqlstr += ("(" + fields + ") ")
        sqlstr += "values "
        sqlstr += ("(" + values + ")")
        
        for val in kwargs['value']:
            values = ''
            for v in val:
                if len(values)>0:
                    values+=","
                
                if val[v] == None:
                    values += 'null'
                else:
                    values += '\'{0}\''.format(val[v])
            sqlstr += (",(" + values + ")")
        
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteReplace(self,**kwargs):
        sqlstr = 'replace into '
        if not 'table' in kwargs:
            raise AddError('Add Has No Table')
        sqlstr += ("`" + kwargs['table'] + "`")
        if not 'value' in kwargs:
            raise AddError('Add Has No Value')
        fields = ""
        values = ""
        firstvalue = kwargs['value'].pop(0)
        for v in firstvalue:
            if len(fields) > 0:
                fields+=","
                values+=","
            fields += r"`{}`".format(v)
            if firstvalue[v] == None:
                values += 'null'
            else:
                values += '\'{0}\''.format(firstvalue[v])
        sqlstr += ("(" + fields + ") ")
        sqlstr += "values "
        sqlstr += ("(" + values + ")")
        
        for val in kwargs['value']:
            values = ''
            for v in val:
                if len(values)>0:
                    values+=","
                if val[v] == None:
                    values += 'null'
                else:
                    values += '\'{0}\''.format(val[v])
            sqlstr += (",(" + values + ")")
        
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteDelete(self,**kwargs):
        sqlstr = "delete from "
        if not 'table' in kwargs:
            raise DeleteError('Delete Has No Table')
        sqlstr += (kwargs['table'] + ' ')
        if 'where' in kwargs:
            sqlstr += ("where " + kwargs['where'])
        #log here

        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data
    
    def excuteUpdate(self,**kwargs):
        sqlstr = "update "
        if not 'table' in kwargs:
            raise DeleteError('Update Has No Table')
        sqlstr += (kwargs['table'] + ' ')
        if not 'value' in kwargs:
            raise DeleteError('Update Has No Value')
        sqlstr += "set "
        setstr = ""
        for val in kwargs['value']:
            for v in val:
                
                if len(setstr) > 0:
                     setstr += ','
                if val[v] == None:
                    
                    setstr += r"`{}` = null".format(v)
                else:
                    setstr += r"`{}` = '{}'".format(v,val[v])
    
            sqlstr += setstr
    
        if 'where' in kwargs:
            sqlstr += (" where " + kwargs['where'])
        #log here
        #print(sqlstr)
        db0 = self.excuteSql(sqlstr)
        data = []
        for d in db0:
            data.append(d)
        return data



from pyDes import des, CBC, PAD_PKCS5
import hashlib
import os
import binascii
import time
 
# 秘钥
def des_encrypt(s,secret_key):
    """
    DES 加密
    :param s: 原始字符串
    :return: 加密后字符串，16进制
    """
    iv = secret_key
    k = des(secret_key, CBC, iv, pad=None, padmode=PAD_PKCS5)
    en = k.encrypt(s, padmode=PAD_PKCS5)
    return binascii.b2a_hex(en)
 
 
def des_descrypt(s,secret_key):
    """
    DES 解密
    :param s: 加密后的字符串，16进制
    :return:  解密后的字符串
    """
    iv = secret_key
    k = des(secret_key, CBC, iv, pad=None, padmode=PAD_PKCS5)
    de = k.decrypt(binascii.a2b_hex(s), padmode=PAD_PKCS5)
    return de

def getfilemd5(filepath):
    if not os.path.isfile(filepath):
        raise Exception("Get MD5 error,it is not a file")
    f = open(filepath,'rb')
    filehash = hashlib.md5()
    
    while True:
        bs = f.read(8096)
        if not bs:
            break
        filehash.update(bs)
    f.close()
    return filehash.hexdigest()

if __name__ == '__main__':
    md5 = getfilemd5('c:\\360base.dll')

    
    print(e-s)


from encription.encription import getfilemd5,des_descrypt,des_encrypt
import os
import random


class FileInfo:
    def __init__(self,path,name):
        self._name = name
        self._dir = DirInfo(path)
        pass
    
    @property
    def FullPath(self):
        return os.path.join(self._dir.Path,self._name)
    
    @property
    def Directory(self):
        return self._dir
    
    @property
    def Name(self):
        return self._name
    
    @property
    def Exists(self):
        return True if os.path.exists(self.FullPath) else False
        
    def getMd5(self):
        rtn = ""
        
        if self.Exists():
            rtn = getfilemd5(self.FullPath)
        return rtn
    
    @property
    def Size(self):
        rtn = 0
        if self.Exists:
            rtn = os.path.getsize(self.FullPath)
        return rtn
    
    def creat(self):
        f = open(self.FullPath,"ab")
        f.close()
        pass
    
    def getFileStream(self,start,batch = 1000):
        rtn = {'success':False}
        if not self.Exists:
            return rtn
        file = open(self.FullPath,'rb')
        
        file.read(start)
        data = file.read(batch)
        
        surplus = len(file.read())
        
        rtn['data'] = data
        rtn['surplus'] = surplus
        
        rtn['success'] = True
        file.close()
        return rtn
    
    def setFileStream(self,start,bs):
        rtn = {'success':False}
        if not self.Directory.Exists:
            self.Directory.creat()
        
        if not self.Exists:
            self.creat()
        
        file = open(self.FullPath,"rb+")
        cur = len(file.read())
        if start != cur:
            
            rtn['err'] = "file size not match with start"
        else:
            batch = file.write(bs)
            rtn['success'] = True
        file.close()
        size = self.Size
        rtn['size'] = size
        return rtn

class DirInfo:
    def __init__(self,path):
        self._path = path
    
    @property
    def Path(self):
        return self._path
    
    @property
    def Exists(self):
        return True if os.path.exists(self._path) else False
    
    def getFilesName(self):
        rtn = {'exists':False}
        if self.Exists:
            files = os.listdir(self._path)
            fs = [file for file in files if os.path.isfile(self._path+file)]
            rtn['exists'] = True
            rtn['files'] = fs
        return rtn
        
    def getInfo(self,file = True,dir = True,belong = True,choise = [],filter = []):
        rtn = {"exists":False}
        
        if os.path.exists(self._path):
            rtn["exists"] = True
            rtn["dirs"] = []
            rtn["files"] = []
            for root,dirs,files in os.walk(self._path):
                for dir in dirs:
                    if root != self._path:
                        continue
                    rtn["dirs"].append(dir)
                for file in files:
                    if root != self._path:
                        continue
                    ext = file.split('.')[-1]
                    if len(filter) > 0:
                        if ext in filter:
                            continue
                    if len(choise) > 0:
                        if not (ext in choise):
                            continue
                    rtn["files"].append(file)
        return rtn
    
    def getFilesInfo(self,start=0,**kwargs):
        batch = 200
        if 'Size' in kwargs:
            batch = batch//2
        if 'Md5' in kwargs:
            batch = batch//4

        rtn = {'start':start}
        
        files = self.getFilesName()
        rtn['filecount'] = len(files)
        if start > len(files)-1:
            return rtn
        end = start + batch
        
        if (start + batch) >= len(files):
            end = len(files)

        fs = []
        rtn['files'] = fs

        for filename in files[start:end]:
            finfo = {'Name':filename}
            fi = FileInfo(self._path,filename)
            if 'Size' in kwargs:
                finfo['Size'] = fi.getSize()
            if 'Md5' in kwargs:
                finfo['Md5'] = fi.getMd5()
            finfo = {'Name':filename,'Md5':md5,'Size':size}
            fs.append(finfo)
        
        return rtn
    
    def creat(self):
        if not self.Exists:
            os.mkdir(self._path)



def setDict(dictionary,keys,value):
	dic = dictionary
	for i in range(len(keys)-1):
		if keys[i] not in dic:
			dic[keys[i]] = {}
		dic = dic[keys[i]]
	dic[keys[-1]] = value
	pass

def getDict(dictionary,*keys,**kwargs):
	dic = dictionary
	rtn = None
	if "default" in kwargs:
		rtn = kwargs["default"]
	for key in keys:
		try:
			dic = dic[key]
		except:
			return rtn
	
	return dic



if __name__ == '__main__':
	from IutyLib.coding.unittest import *
	a = {'a':11,'b':{"b0":112,"b1":"ascii"}}
	i0 = getDict(a,"a")
	assertEqual(i0,11)

	i1 = getDict(a,"b","b0")
	assertEqual(i1,112)

	i3 = getDict(a,"b","b1",1)
	assertEqual(i3,'s')

	i4 = getDict(a,"c")
	assertEqual(i4,None)

	i5 = getDict(a,"c",default="no data")
	assertEqual(i5,"no data")



import datetime
class Process:
	def __init__(self,title,tasks):
		self.starttime = datetime.datetime.now()
		self.title = title
		self.total = len(tasks)

	def showInConsole(self,remain,**kwargs):
		if self.total == 0:
			print('show process error,tasks must not zero')
		process = round(100.0 * (self.total-remain)/self.total,2)
		str_args = ''
		if len(kwargs)>0:
			for arg in kwargs:
				str_args = str_args + "\t@" + arg + ':{0}'.format(kwargs[arg])
		if process > 0:
			endtime = self.starttime + (datetime.datetime.now() - self.starttime)/(process/100.0)
			endstr = endtime.strftime('%Y-%m-%d %H:%M:%S')
		else:
			endstr = '-'
		print('\r'+' '*len(str_args)*2,end='')
		print('\r{0} process -> {1}% will end - > {2} {3}'.format(self.title,process,endstr,str_args),end='')
		if remain == 0:
			print('')
			
def showProcessInConsole(title,starts,remain,total,**kwargs):
	if total == 0:
		print('show process error,tasks must not zero')
	process = round(100.0 * (total - remain)/total,2)
	str_args = ''
	if len(kwargs) > 0:
		for arg in kwargs:
			str_args = str_args + "\t@" + arg + ':{0}'.format(kwargs[arg])
	if process > 0:
		endtime = starts + (datetime.datetime.now() - starts)/(process/100.0)
		endstr = endtime.strftime('%Y-%m-%d %H:%M:%S')
	else:
		endstr = '-'
	print('\r'+' '*50,end='')
	print('\r{0} process -> {1}% will end - > {2} {3}'.format(title,process,endstr,str_args),end='')
	if remain == 0:
		print('')


def showDict(dictionary,index = 0):
	if type(dictionary) != dict:
		print('\t'*(index+1),end='')
		print(dictionary)
		return
	print('\t'*index + '{')
	for dic in dictionary:
		print('\t'*(index+1),end='')
		print(dic,end='')
		print(':')
		showDict(dictionary[dic],index+1)
	print('\t'*index + '}')
	pass

def showTime():
	print('\r'+' '*60,end='')
	print('\r'+datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),end='')

def testShowDict():
	dic = {'root':{'item1':{'item11':11,'item12':12},'item2':0}}
	showDict(dic)
	pass

def showInOneLine(text):
	print("\r" + " "*int(1.1*len(text)),end='')
	print("\r"+text,end='')
	pass

if __name__ == '__main__':
	testShowDict()
	
		




"""
import os,sys,time,json,datetime
from IutyLib.coding.asserts import assertImport

passed = "passed"
skiped = "skiped"
failed = "failed"

class BaseTest:
    _testmode = False
    
    #_modules_ = []
    
    _cases_ = []
    msg = ""
    mustpass = []
    pass

def setup(module,reload=True):
    #if modulepath ins_MethodTest._modules_
    abspath = os.path.abspath(module)
    '''
    if abspath in ins_MethodTest._modules_:
        if reload:
            importlib.reload(module)
            print(r"reload [{}] successed!".format(module))
        return
    '''
    try:
        m = assertImport(module)
        print(r"import [{}] successed！".format(module))
        #ins_MethodTest._modules_.append(abspath)
        return m
    except AssertionError as err:
        print(err)
    
    #print(ins_MethodTest._modules_)
    pass

def case(msg = "have not set message yet",mustpass=[]):
    ins_MethodTest.msg = msg
    ins_MethodTest.mustpass = mustpass
    return testfuncwrapper
    
def testfuncwrapper(func):
    if ins_MethodTest._testmode:
        ins_MethodTest._cases_.append(
        {
            'file':func.__globals__['__file__'],
            'name':func.__name__,
            'func':func,
            'msg':ins_MethodTest.msg,
            "mustpass":ins_MethodTest.mustpass
        })
    
    return func

def test_method(result,testcase):
    
    file = testcase["file"]
    name = testcase["name"]
    func = testcase["func"]
    mustpass = testcase["mustpass"]
    if not (file in result["cases"]):
        result["cases"][file] = {}
    
    if name in result["cases"][file]:
        return True
    
    for item in mustpass:
        #不可跨文件增加约束
        if not (item in result["cases"][file]):
            print("detect mustpass case [{}] when test in {}".format(item,name))
            return False
        else:
            if result["cases"][file][item]["result"] != "passed":
                result["cases"][file][name] = {}
                report = result["cases"][file][name]
                report["result"] = skiped
                report["message"] = "skiped because dependence {} is not pass the test".format(item)
                return True
    print('-'*20)
    
    result["cases"][file][name] = {}
    report = result["cases"][file][name]
    cs = time.time()
    try:
        func()
        print("{} test passed".format(name))
        report["result"] = "passed"
            
    except AssertionError as err:
        
        print("[line:{}] {} test failed! ({})".format(err.__traceback__.tb_next.tb_lineno,name,str(err)))
        report["result"] = "failed"
        report["message"] = "[line:{}]:{}".format(err.__traceback__.tb_next.tb_lineno,str(err))
    ce = time.time()
    report["spend"] = round(ce-cs,3)
    return True



def test_process(resultpath=r"./",testlimit = 10,post = None):
    rtn = {"spend":0.0,"cases":{}}
    gs = time.time()
    
    
    for i in range(testlimit):
        alltested = True
        for testcase in ins_MethodTest._cases_:
            result = test_method(rtn,testcase)
            alltested &= result
        if alltested:
            break
    
    ge = time.time()
    rtn["spend"] = round(ge-gs,3)
    
    now = datetime.datetime.now()
    timestamp = datetime.datetime.strftime(now,"%y%m%d_%H%M%S")
    
    if resultpath:
        try:
            resultfile = os.path.join(resultpath,timestamp+".json")
            with open(resultfile,'w') as f:
                json.dump(rtn,f,indent = 4)
        except Exception as err:
            print("Save test result failed")
            print(err)
        
    if post:
        raise Exception("No post func here")
    pass
    
    



class MethodTest(BaseTest):
    
    pass


    
ins_MethodTest = MethodTest()

"""

_testlist_ = {}

class TestOption:
    def __init__(self):
        self.skip = False
        self.mustpass = False
        pass



def unittest(func):
    if not func in _testlist_:
        _testlist_[func] = TestOption()
    return func


def skip(func):
    if func in _testlist_:
        _testlist_[func].skip = True
    pass

def mustpass(func):
    if func in _testlist_:
        _testlist_[func].mustpass = True
    pass

def businesstest(func):
    if not func in _testlist_:
        _testlist_[func] = TestOption()
    return func



class SqlError(BaseException):
    pass

class TableError(BaseException):
    pass

class QueryError(BaseException):
    pass

class AddError(BaseException):
    pass

class DeleteError(BaseException):
    pass

class UpdateError(BaseException):
    pass


import datetime
def getSerialItem(serial,index,key=0,asIndex = False):
	if len(serial) == 0:
		return None
		
	start = 0
	end = len(serial)
	
	while True:
		
		cur = (end-start)//2 + start
		
		if serial[cur][key] == index:
			if asIndex:
				return cur
			else:
				return serial[cur]
		if (end == start) | (end == (start+1)):
			return None
		if serial[cur][key] < index:
			start = cur
		
		else:
			end = cur
		#print('{},{}-{}'.format(start,end,index))
	pass
	


import os
import py_compile

from IutyLib.coding.tdd import unittest
from IutyLib.coding.asserts import assertTrue


def compile_file(spath,sdir,tdir):
    if not spath.endswith(".py"):
        return
    dir_target = os.path.dirname(spath)
    if not os.path.exists(dir_target):
        os.mkdir(dir_target)
    tpath = str.replace(spath,sdir,tdir)
    tpath += "c"
    py_compile.compile(spath,tpath)
    pass

def compile_list(sdir,tdir):
    abs_sdir = os.path.abspath(sdir)
    abs_tdir = os.path.abspath(tdir)
    for r,ds,fs in os.walk(sdir):
        for f in fs:
            fpath = os.path.abspath(os.path.join(r,f))
            compile_file(fpath,abs_sdir,abs_tdir)
            print(r"compile file "+fpath)
    pass

@unittest
def test_complie_list():
    compile_list(r"./IutyLib/coding/",r"./IutyLib/coding/__pycache__/")
    assertTrue(os.path.exists(r"./IutyLib/coding/__pycache__/compile.pyc"),"compile compile.py complete")
    pass

if __name__ == "__main__":
    #compile_file(r"./1/ttt.py",r"./1",r"./2")
    compile_list(r"./1",r"./2")


import configparser,json,os
def getPickleData(path,isDictionary = False):
    """
    help pass
    """
    import os,pickle
    if os.path.exists(path):
        file = open(path,'rb')
        rtn = pickle.load(file)
        file.close()
        return rtn
    if isDictionary:
        return {}
    return []

def savePickleData(path,memery):
    """
    help pass
    """
    import os,pickle
    f = open(path,'wb')
    pickle.dump(memery,f)
    f.close()
    
class Config:
    def __init__(self,path="./Config/Config.conf"):
        self._path = path
        self._config = configparser.ConfigParser()
        self._config.read(self._path)
        pass
        
    def get(self,session,key):
        rtn = None
        try:
            rtn = self._config.get(session,key)
        except Exception as err:
            a = 1
        return rtn
    
    def set(self,session,key,val):
        if not self._config.has_section(session):
            self._config.add_section(session)
        self._config.set(session,key,val)
        
        self.save()
        pass
    
    def save(self):
        with open(self._path,'w') as f:
            self._config.write(f)
        self._config.read(self._path)
        pass
    
    def getSections(self):
        
        rtn = []
        try:
            rtn = self._config.sections()
        except Exception as err:
            print(str(err)+"in Config getSections")
        return rtn
    
    def getOptions(self,section):
        rtn = []
        try:
            rtn = self._config.options(section)
        except Exception as err:
            print(str(err)+"in Config getOption")
        return rtn
    
    def rmSection(self,section):
        self._config.remove_section(section)
        self.save()
        pass
    
    def rmOption(self,section,option):
        self._config.remove_option(section,option)
        self.save()
        pass
    
    def copy(self,target,section,key,defaultvalue):
        v = self._config.get(section,key)
        if not v:
            v = defaultvalue
        try:
            target.set(section,key,v)
        except Exception as err:
            print(err+"in Config copy")
        pass

class JConfig:
    """
    path:[string] config path
    default:[dict] default value for config
    reset:[bool] delete local config file before load
    """
    def __init__(self,path="./Config/Config.json",default = {},reset=False):
        self._path = path
        self._config = default
        if reset:
            os.remove(self._path)
        self.load()
        pass
    
    def load(self):
        try:
            if os.path.exists(self._path):
                with open(self._path,'r') as f:
                    self._config = json.load(f)
        except:
            print("load json error")
        pass
    
    def save(self):
        with open(self._path,'w') as f:
            json.dump(self._config,f,indent=4)
        self.load()
        pass
    
    def get(self):
        return self._config
    
    pass
    
if __name__ == "__main__":
    cfg = JConfig(default={'app':"test app"})
    #cfgvalue = cfg.get()
    #cfgvalue["app"] = "appname name"
    #cfgvalue["setting"] = {'port':3378,'ip':"127.0.0.1"}
    cfg.save()


import tensorflow as tf

# -----------------------------------------------------------------------------
# loss计算
# 传入参数：logits，网络计算输出值。labels，真实值，在这里是0或者1
# 返回参数：loss，损失值
def losses(logits, labels):
    print(logits)
    print(labels)
    with tf.variable_scope('loss') as scope:
        cross_entropy = tf.nn.sparse_softmax_cross_entropy_with_logits(logits=logits, labels=labels,name='xentropy_per_example')
        loss = tf.reduce_mean(cross_entropy, name='loss')
        tf.summary.scalar(scope.name + '/loss', loss)
    return loss


# --------------------------------------------------------------------------
# loss损失值优化
# 输入参数：loss。learning_rate，学习速率。
# 返回参数：train_op，训练op，这个参数要输入sess.run中让模型去训练。
def trainning(loss, learning_rate):
    with tf.name_scope('optimizer'):
        optimizer = tf.train.AdamOptimizer(learning_rate=learning_rate)
        global_step = tf.Variable(0, name='global_step', trainable=False)
        train_op = optimizer.minimize(loss, global_step=global_step)
    return train_op


# -----------------------------------------------------------------------
# 评价/准确率计算
# 输入参数：logits，网络计算值。labels，标签，也就是真实值，在这里是0或者1。
# 返回参数：accuracy，当前step的平均准确率，也就是在这些batch中多少张图片被正确分类了。
def evaluation(logits, labels):
    with tf.variable_scope('accuracy') as scope:

        correct = tf.nn.in_top_k(logits, labels, 1)
        correct = tf.cast(correct, tf.float16)
        accuracy = tf.reduce_mean(correct)
        tf.summary.scalar(scope.name + '/accuracy', accuracy)
    return accuracy


import json
def isIns(obj):
    return any([isinstance(obj,int) , isinstance(obj,str) , isinstance(obj,float)])
    
def extract(obj):
    rtn = None
    if isIns(obj):
        rtn = obj
    
    else:
        
        if type(obj) is dict:
            rtn = obj
            its = obj
        elif type(obj) is list:
            rtn = obj
            its = range(len(obj))
        else:
            rtn = obj.__dict__
            its = obj.__dict__
        for it in its:           
            if not isinstance(rtn[it],type):                
                rtn[it] = extract(rtn[it])
    return rtn

def inject(obj,json):
    
    rtn = None
    if isIns(json):
        rtn = json
    else:
        rtn = obj
        if type(obj) is dict:
            
            for it in json:
                obj[it] = inject(None,json[it])
                
            
        elif type(obj) is list:
            for it in range(len(json)):
                v = inject(None,json[it])
                if it < len(obj):
                    obj[it] = v
                else:
                    obj.append(v)
        else:
            for d in json:
                
                obj.__dict__[d] = inject(obj.__dict__[d],json[d])
    return rtn

class Model:

            
    
    def toConfig(self):
        
        return extract(self)
    
    def fromConfig(self,config):
        inject(self,config)


if __name__ == "__main__":
    class B:
        def __init__(self):
            self._B = "my B"
            self.__t__ = {"t1":13}
            self._t_ = [1]

    class A(Model):
        def __init__(self):
            self.A = 10
            
            self.B = B()
            pass
    
    a = A()
    a.B._B = "changed B"
    a.B.__t__["append"] = "vv13"
    a.B._t_.append(10)
    config = a.toConfig()
    print(config)
    a1 = A()
    a1.fromConfig(config)
    print(a1.B._B)
    print(a1.B.__t__)
    print(a1.B._t_)

    


from abc import abstractmethod
import datetime,os,shutil
import IutyLib.commonutil.config as config
import IutyLib.commonutil.dict
from IutyLib.file.log import SimpleLog
from IutyLib.show.console import Process as ShowProcess


def initPath(file):
	envpath = os.path.dirname(file)
	if not envpath in os.sys.path:
		os.sys.path.append(envpath)


class ProxyBase:
	def __init__(self,name = None):
		configpath = ".//config//"
		if name == None:
			self.name = self.__class__.__name__
			self.__configpath__ = configpath + self.__class__.__name__ + ".pkl"
		else:
			self.name = name
			self.__configpath__ = configpath + name + ".pkl"
				
		if not os.path.exists(self.__configpath__):
			self.__config__ = {}
			self.initConfig()
		self.__config__ = config.getPickleData(self.__configpath__)


		if "logpath" in self.__config__:
			self.log = SimpleLog(self.__config__["logpath"])
		else:
			self.log = SimpleLog()

		self.__cmds__ = {}

		self.initResource()

		pass

	@abstractmethod
	def initConfig(self):
		#user should set default config here
		pass

	def setConfig(self,keys,value):
		commonutil.setDict(self.__config__,keys,value)
		self.saveConfig()
		
	def saveConfig(self):
		config.savePickleData(self.__configpath__,self.__config__)

	@abstractmethod
	def initResource(self):
		#init class resource here
		pass

	@abstractmethod
	def initCmd(self):
		#regedit cmd here
		pass

	def excuteCmd(self,keyword,**kwargs):
		if keyword in self.__cmds__:
			return self.__cmds__[keyword].__call__(**kwargs)
		return None

class TaskProxy(ProxyBase):
	def __init__(self,name = None):
		ProxyBase.__init__(self,name)
		self.__taskname__ = None
		self.__tasks__ = []
		self.OnTaskFinished = None
		
	@property
	def Finish(self):
		return len(self.__tasks__) == 0
	
	@property
	def TaskName(self):
		return self.__taskname__
	
	def getTaskStatus(self,title):
		return self.__config__['tasks'][title]
	
	def setTaskFinishStatus(self,value):
		self.setConfig(['tasks',self.taskname],value)
		self.taskname = None
	
	def initTask(self,title,tasks):
		self.taskname = title
		self.__tasks__ += tasks
		self.setConfig(['tasks',title],datetime.datetime.now())
		self.p_show = ShowProcess(title,self.__tasks__)
	
	@abstractmethod
	def runTask(self,arg):
		#ruturn the param of task
		pass

	def excuteTask(self):
		if len(self.__tasks__) == 0:
			return
		task = self.__tasks__.pop(0)
		
		kwargs = self.runTask(task)
		
		if kwargs == None:
			kwargs = {}
		self.p_show.showInConsole(len(self.__tasks__),**kwargs)

#abort
class ServiceCommand:
	def __init__(self):
		self.__cmds__ = {}
		self.__subobj__ = []
		
		self.initCmd()
		
	@abstractmethod
	def initCmd(self):
		#init cmd format here
		pass
	
	def excuteCmd(self,keyword,**kwargs):
		if keyword in self.__cmds__:
			print('service excute command [{0}]...,'.format(keyword))
			return self.__cmds__[keyword].__call__(**kwargs)
		else:
			for sub in self.__subobj__:
				result = sub.excuteCmd(keyword,**kwargs)
				if result != None:
					print('{0} excute command [{1}]...'.format(keyword))
					return result
		return None

	def invokeCmd(self,cmdstring):
		rtn = {'success': False}
		temp0 = cmdstring.split(',')
		keyword = temp0[0]
		kwargs = {}
		if len(temp0) > 0:
			
			for i in range(1,len(temp0)):
				print(temp0)
				temp1 = temp0[i].split('=')
				if len(temp1) != 2:
					print('kwarg error in [{0}]'.format(temp0[i]))
					return rtn
				kwargs[temp1[0]] = temp1[1]
		print('receive command [{0}]...,'.format(cmdstring))
		rtn['success'] = True
		result = self.excuteCmd(keyword,**kwargs)
		if result is None:
			print('service have no result...')
		else:
			print('result here...:')
			for r in result:
				print('\t{0}->{1}'.format(r,result[r]))
			print('result print finished...')
		rtn['data'] = result
		return rtn
	
	def cmd(func):
		self.__cmds__[func.__name__] = func



	
	
		
'''

'''
class ServiceFunc:
	def __init__(self):
		self.__cmds__ = {}
		self.__blls__ = {}
		pass
		
	
	def cmd(func):
		servicefunc.__cmds__[func.__name__.lower()] = func
		return func

	def invokeCmd(obj,cmdstring):
		rtn = {'success': False}
		cmdstring = cmdstring.lower()
		temp0 = cmdstring.split(',')
		keyword = temp0[0]
		kwargs = {}
		if len(temp0) > 0:
			for i in range(1,len(temp0)):
				#print(temp0)
				temp1 = temp0[i].split('=')
				if len(temp1) != 2:
					print('kwarg error in [{0}]'.format(temp0[i]))
					return rtn
				kwargs[temp1[0]] = temp1[1]
		print('receive command [{0}]...,'.format(cmdstring))
		rtn['success'] = True
		
		if keyword == 'getcmds':
			rtn['result'] = list(servicefunc.__cmds__.keys())
			print(rtn)
			return rtn
		
		result = None
		if keyword in servicefunc.__cmds__:
			result = servicefunc.__cmds__[keyword].__call__(obj,**kwargs)
		
		print(result)
		print('result print finished...')
		rtn['result'] = result
		return rtn
	
	def bll(func):
		servicefunc.__blls__[func.__name__.lower()] = func
		return func
	
	def invokeBll(obj,**kwargs):
		rtn = {'success':False,'err':'service has no this func'}
		funcname = kwargs['func'].lower()
		if not funcname in servicefunc.__blls__:
			return rtn
		kwargs.pop('func')
		result = servicefunc.__blls__[funcname].__call__(obj,**kwargs)
		return result

class WebServiceFunc:
	def __init__(self):
		self.__blls__ = {}
		self._servicetype = 'WebService'
		pass
	
	
	def bll(func):
		servicefunc.__blls__[func.__name__.lower()] = func
		return func
	
	def invokeBll(obj,**kwargs):
		rtn = {'success':False,'err':'service has no this func'}
		funcname = kwargs['func'].lower()
		if not funcname in servicefunc.__blls__:
			return rtn
		kwargs.pop('func')
		result = servicefunc.__blls__[funcname].__call__(obj,**kwargs)
		return result

servicefunc = ServiceFunc()
webservicefunc = WebServiceFunc()

class ServiceBase:
	def __init__(self):
		self.__configpath__ = './/config//config.pkl'
		if os.path.exists(self.__configpath__):
			self.config = {}
			self.initConfig()
		self._servicetype = 'Service'
		self._run = False
		self.config = config.getPickleData(self.__configpath__)

		self.log = SimpleLog('.//logs//HostInfo//')
		
		self.initResource()
		
		pass

	@abstractmethod
	def initConfig(self):
		#set config default value here
		pass

	def setConfig(self,keys,value):
		commonutil.setDict(self.config,keys,value)
		config.savePickleData(self.__configpath__,self.config)

	@abstractmethod
	def initResource(self):
		#init Proxy Resource here
		pass
	
	@abstractmethod
	def start(self,**kwargs):
		return True
	
	@ServiceFunc.cmd
	def doStart(self,**kwargs):
		self._run = self.start(**kwargs)
		return {'run':self._run}
		
	
	@abstractmethod
	def stop(self,**kwargs):
		return False
	
	@ServiceFunc.cmd
	def doStop(self,**kwargs):
		self._run = self.stop(**kwargs)
		return {'run':self._run}
		
	@ServiceFunc.cmd
	def getStatus(self,**kwargs):
		return {'run':self._run}
	
	pass

class InitHelper:
	def __init__(self):
		self.basepath = './/'
		
		self.servicepath = self.basepath + 'Service//'
		self.proxypath = self.servicepath + 'Proxys//'
		self.logpath = self.servicepath + 'Logs//'
		self.configpath = self.servicepath + 'Config//'
		
		self.testpath = self.basepath + 'Test//'
		self.testproxypath = self.testpath + 'Proxys//'
		self.testlogpath = self.testpath + 'Logs//'
		self.testconfigpath = self.testpath + 'Config//'
		self.testresultpath = self.testpath + 'Result//'
		
		self.templatepath = os.environ['PythonPath'] + '//template//'
	
	def initFrameWork(self):
		if not os.path.exists(self.basepath):
			os.mkdir(self.basepath)
		
		if not os.path.exists(self.servicepath):
			os.mkdir(self.servicepath)
		
		if not os.path.exists(self.proxypath):
			os.mkdir(self.proxypath)
		
		if not os.path.exists(self.logpath):
			os.mkdir(self.logpath)
		
		if not os.path.exists(self.configpath):
			os.mkdir(self.configpath)
		
		if not os.path.exists(self.testpath):
			os.mkdir(self.testpath)
		
		if not os.path.exists(self.testproxypath):
			os.mkdir(self.testproxypath)
		
		if not os.path.exists(self.testlogpath):
			os.mkdir(self.testlogpath)
		
		if not os.path.exists(self.testconfigpath):
			os.mkdir(self.testconfigpath)
		
		if not os.path.exists(self.testresultpath):
			os.mkdir(self.testresultpath)
		
		if not os.path.exists(self.servicepath + 'Service.py'):
			if os.path.exists(self.templatepath+'Service.py'):
				shutil.copy(self.templatepath+'Service.py',self.servicepath + 'Service.py')
			else:
				print('Warning:Service template not exists')
		
		if not os.path.exists(self.servicepath + '__run__.py'):
			if os.path.exists(self.templatepath+'__run__.py'):
				shutil.copy(self.templatepath+'__run__.py',self.servicepath + '__run__.py')
			else:
				print('Warning:__run__ template not exists')



def getMA(serial,index,period,p=4):
    if period > index:
        return None
    if index >= len(serial):
        return None
    total = 0.0
    for i in range(index-period+1,index+1):
        total += serial[i][p]
    return total/period


def getHHI(serial,index,period=20,p=2):
    if period > index:
        return None
    if index >= len(serial):
        return None
    
    ri = index - period + 1
    for i in range(index-period+1,index+1):
        if serial[i][p] > serial[ri][p]:
            ri = i
    return ri

def getLLI(serial,index,period=20,p=3):
    if period > index:
        return None
    if index >= len(serial):
        return None
    
    ri = index - period + 1
    for i in range(index-period+1,index+1):
        if serial[i][p] < serial[ri][p]:
            ri = i
    return ri

def getSum(serial,index,period,p=4):
    if period > index:
        return None
    if index >= len(serial):
        return None
    total = 0.0
    for i in range(index-period+1,index+1):
        total += serial[i][p]
    return total

def getStd(serial,index,period = 20,p=4):
    ma = getMa(s,period,index,p)
    std2 = 0.0
    for i in range(index-period+1,index+1):
        
        std2 += (s[i][p]-ma) ** 2
    std = std2 ** 0.5
    return std/period




import time
import requests
import json,os


class WeChat_SMS:
    def __init__(self,corpsecret = '_4vuUmRxg9V5bnb1dZ2QlhHIBiwIL5O7SMvowRX7_WA',agentid = '1000002',partid='0'):
        self._corpid = 'ww97a5a59259fbb43d'#企业ID， 登陆企业微信，在我的企业-->企业信息里查看
        self._corpsecret = corpsecret#自建应用，每个自建应用里都有单独的secret
        self._agentid = agentid #应用代码
        self._partid = partid


    @property
    def CorpId(self):
        return self._corpid
        
    @CorpId.setter
    def CorpId(self,value):
        self._corpid = value
        
    @property
    def CorpSecret(self):
        return _corpsecret
        
    @CorpSecret.setter
    def CorpSecret(self,value):
        self._corpsecret = value
        
    @property
    def AgentId(self):
        return _agentid
        
    @AgentId.setter
    def AgentId(self,value):
        self._agentid = value
        

    def _get_access_token(self):
        url = 'https://qyapi.weixin.qq.com/cgi-bin/gettoken'
        values = {'corpid': self._corpid,'corpsecret': self._corpsecret,}
        req = requests.post(url, params=values)
        data = json.loads(req.text)
        return data["access_token"]

    def get_access_token(self):
        try:
            with open('access_token_{0}.conf'.format(self._agentid), 'r') as f:
                t, access_token = f.read().split()
        except:
            with open('access_token_{0}.conf'.format(self._agentid), 'w') as f:
                access_token = self._get_access_token()
                cur_time = time.time()
                f.write('\t'.join([str(cur_time), access_token]))
                return access_token
        else:
            cur_time = time.time()
            if 0 < cur_time - float(t) < 7200:#token的有效时间7200s
                return access_token
            else:
                with open('access_token_{0}.conf'.format(self._agentid), 'w') as f:
                    access_token = self._get_access_token()
                    f.write('\t'.join([str(cur_time), access_token]))
                    return access_token

    def send_data(self, msg,user='@all'):
        send_url = 'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=' + self.get_access_token()
        send_values = {
            "touser": user,
            "toparty": self._partid,     #设置给部门发送
            "msgtype": "text",
            "agentid": self._agentid,
            "text": {
            "content": msg
            },
            "safe": "0"
        }
        send_msges=(bytes(json.dumps(send_values), 'utf-8'))
        respone = requests.post(send_url, send_msges)
        respone = respone.json()#当返回的数据是json串的时候直接用.json即可将respone转换成字典
        #print (respone["errmsg"])
        return respone["errmsg"]
        
    def send_taskcard(self,msg,user='@all'):
        send_url = 'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=' + self.get_access_token()
        send_values = {
            "touser" : user,
            "toparty" : self._partid,
            #"totag" : "TagID1 | TagID2",
            "msgtype" : "taskcard",
            "agentid" : self._agentid,
            "taskcard" : {
                "title" : "赵明登的礼物申请",
                "description" : "礼品：A31茶具套装<br>用途：赠与小黑科技张总经理",
                "url" : "http://119.3.254.8/todo",
                "task_id" : "task1234",
                "btn":[
                    {
                        "key": "key111",
                        "name": "批准",
                        "replace_name": "已批准",
                        "color":"red",
                        "is_bold": True
                    },
                    {
                        "key": "key222",
                        "name": "驳回",
                        "replace_name": "已驳回"
                    }
                ]
            },
        #"enable_id_trans": 0,
        #"enable_duplicate_check": 0,
        #"duplicate_check_interval": 1800
        "safe":"0"
        }
        send_msges=(bytes(json.dumps(send_values), 'utf-8'))
        respone = requests.post(send_url, send_msges)
        respone = respone.json()#当返回的数据是json串的时候直接用.json即可将respone转换成字典
        print (respone["errmsg"])
        return respone["errmsg"]
    
    def send_markdown(self,msg,user='@all'):
        send_url = 'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=' + self.get_access_token()
        send_values = {
            "touser" : user,
            "toparty" : self._partid,
            #"totag" : "TagID1 | TagID2",
            "msgtype" : "markdown",
            "agentid" : self._agentid,
            "markdown": {
            "content": "您的会议室已经预定，稍后会同步到`邮箱`                 >**事项详情**                 >事　项：<font color=\"info\">开会</font>                 >组织者：@miglioguan                 >参与者：@miglioguan、@kunliu、@jamdeezhou、@kanexiong、@kisonwang                 >                 >会议室：<font color=\"info\">广州TIT 1楼 301</font>                 >日　期：<font color=\"warning\">2018年5月18日</font>                 >时　间：<font color=\"comment\">上午9:00-11:00</font>                 >                 >请准时参加会议。                 >                 >如需修改会议信息，请点击：[修改会议信息](https://work.weixin.qq.com)"
            },
        #"enable_id_trans": 0,
        #"enable_duplicate_check": 0,
        #"duplicate_check_interval": 1800
        "safe":"0"
        }
        send_msges=(bytes(json.dumps(send_values), 'utf-8'))
        respone = requests.post(send_url, send_msges)
        respone = respone.json()#当返回的数据是json串的时候直接用.json即可将respone转换成字典
        print (respone["errmsg"])
        return respone["errmsg"]
        
    def send_textcard(self,msg,user='@all'):
        send_url = 'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=' + self.get_access_token()
        send_values = {
            "touser" : user,
            "toparty" : self._partid,
            #"totag" : "TagID1 | TagID2",
            "msgtype" : "textcard",
            "agentid" : self._agentid,
            "textcard" : {
                "title" : "领奖通知",
                "description" : "<div class=\"gray\">2016年9月26日</div> <div class=\"normal\">恭喜你抽中iPhone 7一台，领奖码：xxxx</div><div class=\"highlight\">请于2016年10月10日前联系行政同事领取</div>",
                "url" : "http://119.3.254.8/todo",
                "btntxt":"更多"
            },
        #"enable_id_trans": 0,
        #"enable_duplicate_check": 0,
        #"duplicate_check_interval": 1800
        "safe":"0"
        }
        send_msges=(bytes(json.dumps(send_values), 'utf-8'))
        respone = requests.post(send_url, send_msges)
        respone = respone.json()#当返回的数据是json串的时候直接用.json即可将respone转换成字典
        print (respone["errmsg"])
        return respone["errmsg"]


if __name__ == '__main__':
    wx = WeChat_SMS()
    #wx.send_data(msg="服务崩了，你还在这里吟诗作对？")
    wx.send_textcard(msg = "")
    #以下是添加对日志的监控
    # srcfile = u"G:/123.txt"
    # file = open(srcfile)
    # file.seek(0, os.SEEK_END)
    # while 1:
    #     where = file.tell()
    #     line = file.readline()
    #     if not line:
    #         time.sleep(1)
    #         file.seek(where)
    #     else:
    #         print(line)
    #         wx.send_data(msg=line)



from IutyLib.stock.files import BlockFile,VLevelFile,DailyFile
from IutyLib.stock.qualification import VLs,VHs

class Block:
	from abc  import abstractmethod
	def __init__(self,code,cqcx = False):
		self.qfile = BlockFile(code,cqcx)
		if cqcx:
			self.dfile = CqcxDailyFile(code)
		else:
			self.dfile = DailyFile(code)
		self.d_data = self.dfile.getData()
		self.code = code
		q_data = self.qfile.getData()
		self.q_data = {}
		self.blockvalue = None
		if len(q_data) == 0:
			return
		for q in q_data:
			self.q_data[q[0]] = q[1] 
			
		pass
	
	def getNextIndex(self):
		if len(self.q_data) == 0:
			return 0
		sum = 0
		for v in self.q_data:
			sum += self.q_data[v]
		times = sum/2
		if times == len(self.d_data):
			return -1
		else:
			return int(times)
		
		
	def calcAppends(self):
		last = self.getNextIndex()
		if last == -1:
			return last
		d_data = self.d_data
		q_data = self.q_data
		
		for i in range(last,len(d_data)):
			lv = d_data[i][2]
			hv = d_data[i][3]
			if not lv in q_data:
				q_data[lv] = 0
			if not hv in q_data:
				q_data[hv] = 0
			q_data[lv] += 1
			q_data[hv] += 1
		return last
	
	def writeAppends(self):
		last = self.calcAppends()
		if last == -1:
			return
		
		appends = []
		for q in self.q_data:
			appends.append((q,self.q_data[q]))
		self.qfile.appendData(appends)
		pass
	
	def getBlockValues(self):
		sortedkeys = sorted(self.q_data.keys())
		sorteddict = {}
		for key in sortedkeys:
			sorteddict[key] = 0
			for key0 in self.q_data:
				if (abs(key0 - key))/key < 0.005:
					sorteddict[key] += self.q_data[key0]
		
		mainvalue = 0
		mainkey = None
		for key in sorteddict:
			if sorteddict[key] > mainvalue:
				mainvalue = sorteddict[key]
				mainkey = key
		
		v_max = max(list(sorteddict.keys()))
		v_min = min(list(sorteddict.keys()))
		valuedict = {}
		for i in range(0,100):
			up0 = 1.05**i
			up1 = 1.05**(i+1)
			down0 = 0.95**i
			down1 = 0.95**(i+1)
			upvalue = None
			upcount = 0
			downvalue = None
			downcount = 0
			for key in sorteddict:
				if (key>=mainkey*up0) & (key<mainkey*up1):
					if sorteddict[key] > upcount:
						upvalue = key
						upcount = sorteddict[key]
				if (key<=mainkey*down0) & (key>mainkey*down1):
					if sorteddict[key] > downcount:
						downvalue = key
						downcount = sorteddict[key]
			if upvalue!= None:
				valuedict[upvalue] = upcount
			if downvalue != None:
				valuedict[downvalue] = downcount
		
		sortedblockkeys = sorted(valuedict.keys())
		sortedblockvalues = {}
		for key in sortedblockkeys:
			sortedblockvalues[key] = valuedict[key]
		return sortedblockvalues
	
	def getValues(self):
		if self.blockvalue == None:
			self.blockvalue = self.getBlockValues()
		return self.blockvalue
	
class VLevel:
	from abc  import abstractmethod
	def __init__(self,code,cqcx = False):
		self.qfile = VLevelFile(code,cqcx)
		if cqcx:
			self.dfile = CqcxDailyFile(code)
		else:
			self.dfile = DailyFile(code)
		self.d_data = self.dfile.getData()
		self.code = code
		q_data = self.qfile.getData()
		self.q_data = {}
		self.blockvalue = None
		if len(q_data) == 0:
			return
		for q in q_data:
			self.q_data[q[0]] = (q[1],q[2])
			
		pass
	
	def getDataFromDaily(self,item):
		dict_rtn = {}
		last = len(self.d_data)-1
		if last == 0:
			return dict_rtn
		d_data = self.d_data
		level = 1
		while 1:
			if item == 3:
				vs = VLs(self.d_data,last,3,level)
			else:
				vs = VHs(self.d_data,last,2,level)
				#print(vs)
			if len(vs) == 0:
				#no value
				break
			else:
				#value set
				for v in vs:
					#print(v)
					if not v[item] in dict_rtn:
						dict_rtn[v[item]] = (1,0)
						
					dict_rtn[v[item]] = (dict_rtn[v[item]][0],dict_rtn[v[item]][1]+1)
					if dict_rtn[v[item]][0] < level:
						print('rrrr')
						dict_rtn[v[item]] = (level,dict_rtn[v[item]][1])
			level += 1
		return dict_rtn
	
	def mergeUnilateral(self,dict0,range = 0.005):
		for item in dict0:
			itemvalue = dict0[item]
			for item1 in dict0:
				if item1 != item:
					item1value = dict0[item1]
					if (item1value[0] < itemvalue[0]) & (item1 > item * 1-range) & (item1 < item * 1+range):
						dict0[item] = (dict0[item][0],itemvalue[1] + item1value[1])
		pass
	
	def calcAppends(self):
		
		vls = self.getDataFromDaily(3)
		#print(vls)
		vhs = self.getDataFromDaily(2)
		print(vhs)
		
		self.mergeUnilateral(vls)
		self.mergeUnilateral(vhs)
		
		vs = {}
		for item in vls:
			vs[item] = vls[item]
		for item in vhs:
			if item in vs:
				vs[item] = (vs[item][0]+vhs[item][0])
			else:
				vs[item] = (vhs[item][0],0)
			print('ttt')
			print(vs[item])
			vs[item] = (vs[item][0],vs[item][1] + vhs[item][1])
		self.mergeUnilateral(vs)
		self.q_data = vs
		#print(len(self.q_data))
		
	
	def writeAppends(self):
		self.calcAppends()
		
		appends = []
		for q in self.q_data:
			appends.append((q,self.q_data[q][0],self.q_data[q][1]))
		self.qfile.appendData(appends)
		pass
	
	def getLValues(self):
		sortedkeys = sorted(self.q_data.keys())
		sorteddict = {}
		#print(len(sortedkeys))
		for key in sortedkeys:
			b_select = True
			for key0 in self.q_data:
				if key != key0:
					if (key0 < key * 1.05) & (key0 > key * 0.95):
						if self.q_data[key][1] * self.q_data[key][2] < self.q_data[key0][1] * self.q_data[key0][2]:
							b_select = false
							break
			if b_select:
				sorteddict[key] = self.q_data[key]
		return sorteddict
	
	def getValues(self):
		if self.blockvalue == None:
			self.blockvalue = self.getBlockValues()
		return self.blockvalue



class MontecarloCore:
    def __init__(self):
        self._vl_ = []
        self.mt = 0
        self.at = 0
        pass
    
    def add(self,v,r=1):
        self.at += r
        if len(self._vl_) == 0:
            self._vl_.append(0.0)
        if self._vl_[0] > 50000:
            self._vl_.insert(0,0.0)
        self._vl_[0] += v
        pass
    
    def mul(self,v,r=1):
        self.mt += r
        if len(self._vl_) == 0:
            self._vl_.append(1.0)
        if self._vl_[0] > 50000 or self._vl_[0] < 0.0005:
            self._vl_.insert(0,1.0)
        self._vl_[0] *= v
        pass
    
    def getValue(self):
        if (self.at > 0) and (self.mt) == 0:
            rtn = 0.0
            for v in self._vl_:
                rtn += v/self.at
            return rtn
        if (self.mt) > 0 and (self.at) == 0:
            rtn = 1.0
            for v in self._vl_:
                rtn *= v**(1/self.mt)
            return rtn
        return 0.0
    
    pass
    
if __name__ == "__main__":
    import random
    mc = MontecarloCore()
    for i in range(10000000+1):
        x = random.random()
        y = random.random()
        r2=x**2+y**2
        if r2 > 1:
            mc.add(1)
        else:
            mc.add(4)
        
        if i == 100 or i == 1000 or i == 100000 or i == 1000000 or i == 10000000:
            print("step:{}, pi={}".format(i,mc.getValue()))
    pass


from IutyLib.stock.files import QualifyFile, HourFile,QuarterFile, DailyFile, WeeklyFile, CqcxDailyFile,StackFile

'''
Qualify Method
'''
def getIndexOfSerial(Serial,date):
    for data in Serial:
        if data[0] == date:
            return Serial.index(data)
    return -1

def SMA(Pre,Cur,N,M):
#N:Cycle Count
#M:Right Value
    if (M>N):
        return None
    
    v = ((N-M)*Pre + M * Cur)/N
    return v

def EMA(Pre,Cur,N):
    v = (Cur*2 + Pre*(N-1))/(N+1)
    return v

def REF(serial,base,interval):
    index = serial.index(base)
    if index < interval:
        return None
    return serial[index-interval]

def MUL(serial,index):
    default = None
    if index < 4:
        return default
    for i in range(index-4,0,-1):
        if serial[i][6]/serial[i-1][6] > 2.0:
            lev = serial[i-1][4]
            v = 3*(serial[i][4] - lev)/4
            if serial[i][4]<lev:
                return default
            if serial[i+1][4] < lev+v:
                return default
            if serial[i+2][4] < lev+v:
                return default
            if serial[i+3][4] < lev+v:
                return default
            return i
    return default

def WAV(serial,index):
    default = None
    if index < 20:
        return default
    vl2 = VLV(serial,index,5,3)
    if vl2[0] == index:
        return default
    b2 = serial[vl2[0]][4]
    
    vh1 = VHV(serial,vl2[0],5,3)
    if vh1[0] == vl2[0]:
        return default
    a1 = serial[vh1[0]][4]
    
    vl1 = VLV(serial,vh1[0],5,3)
    if vl1[0] == vh1[0]:
        return default
    b1 = serial[vl1[0]][4]
    
    return[(vl1[0],b1),(vh1[0],a1),(vl2[0],b2)]

def TSH(serial,index,threshold):
    h = serial[index][4]
    l = serial[index][4]
    c = serial[index][4]
    delta = 1
    for i in range(index,-1,-1):
        h = max(h,serial[i][2])
        l = min(h,serial[i][3])
        deltah = h - c
        deltal = c - l
        if deltal != 0:
            delta = deltah/deltal
        if (delta > (1 + threshold)) | (delta < (1 - threshold)):
            break
    return (index,delta)

def AMVL(mvl_data,date,lastclose = None,bound = 1.045):
    if len(mvl_data) == 0:
        return []
    m_index = 0
    for data in mvl_data:
        if data[0] > date:
            m_index = mvl_data.index(data)-1
            break
        m_index = len(mvl_data) -1
    serial = mvl_data[:m_index]
    vols = sorted(serial, key = lambda serial:serial[2])
    
    rtn = [mvl_data[m_index]]

def SMVL(mvl_data,date,lastclose = None,bound = 1.045):
    
    if len(mvl_data) == 0:
        return []
    m_index = 0
    for data in mvl_data:
        if data[0] > date:
            m_index = mvl_data.index(data)-1
            break
        m_index = len(mvl_data) -1
    if m_index == 0:
        return []
    
    rtn = [mvl_data[m_index]]
    for i in range(m_index-1,0,-1):
        insert_index = len(rtn)
        b_insert = True
        for data in rtn:
            if data[1] > mvl_data[i][1]:
                if mvl_data[i][1] > data[1]/bound:
                    b_insert = False
                    break
            if data[1] < mvl_data[i][1]:
                if mvl_data[i][1] < data[1]*bound:
                    b_insert = False
                    break
            if data[1] == mvl_data[i][1]:
                b_insert = False
                break
            if data[1] > mvl_data[i][1]:
                if insert_index == len(rtn):
                    insert_index = rtn.index(data)
                    continue
                if data[1] < rtn[insert_index][1]:
                    insert_index = rtn.index(data)
        if b_insert:
            rtn.insert(insert_index,mvl_data[i])
    if not lastclose is None:
        tul = []
        for i in range(1,len(rtn)):
            if rtn[i][1] > lastclose:
                tul.append(rtn[i-1])
                tul.append(rtn[i])
                break
        return tul
    return rtn
    

def LK(serial,index,item):
    default = None
    if index < 20:
        return default
    if len(serial) < index:
        return default
    k = -10000
    for i in range(index-1,0,-1):
        k1 = (serial[index][item] - serial[i][item])/(index-i)
        
        if k1 > k:
            k = k1
            continue
        if k1 <= k:
            #print('k={0},k1={1}'.format(k,k1))
            return (i+1,k)
    return default
    
def HK(serial,index,item):
    default = None
    if index < 20:
        return default
    if len(serial) < index:
        return default
    k = 10000
    for i in range(index-1,0,-1):
        k1 = (serial[index][item] - serial[i][item])/(index-i)
        
        if k1 < k:
            k = k1
            continue
        if k1 >= k:
            #print('k={0},k1={1}'.format(k,k1))
            
            return (i,k)
    return default

def VHV(serial,index,item,length = 7):
    default = (index,serial[index][item])
    source = []
    for i in range(index,0,-1):
        source.append(serial[i][item])
        if len(source) < length:
            continue
        i_rtn = length//2
        if source[i_rtn] == max(source):
            return (i+i_rtn,serial[i+i_rtn][item])
        source.pop(0)
    return default

def VLV(serial,index,item,length = 7):
    default = (index,serial[index][item])
    source = []
    for i in range(index,0,-1):
        source.append(serial[i][item])
        if len(source) < length:
            continue
        i_rtn = length//2
        if source[i_rtn] == min(source):
            return (i+i_rtn,serial[i+i_rtn][item])
        source.pop(0)
    return default

def LH(serial,index,item):
    default = (index,serial[index][item])
    for i in range(index-1,0,-1):
        if (serial[i][item] > serial[i-1][item]) & (serial[i][item] > serial[i+1][item]):
            #if serial[i][item] > serial[index][item]:
            return (i,serial[i][item])
            #else:
            #    return default
    return default

def LL(serial,index,item):
    default = (index,serial[index][item])
    for i in range(index-1,0,-1):
        if (serial[i][item] < serial[i-1][item]) & (serial[i][item] < serial[i+1][item]):
            #if serial[i][item] > serial[index][item]:
            return (i,serial[i][item])
            #else:
            #    return default
    return default

def VL(serial,index,item):
    default = (index,serial[index][item])
    ll1 = LL(serial,index,item)
    if (ll1[0] == index):
        
        return default
    m_index = ll1[0]-1
    while 1:
        ll2 = LL(serial,m_index,item)
        
        if ll2[0] == m_index:
            
            return default
        if ll2[1] > ll1[1]:
            return ll1
        
        ll1 = ll2
        m_index = ll1[0] - 1
    return default
    
def VH(serial,index,item):
    default = (index,serial[index][item])
    
    lh1 = LH(serial,index,item)
    if (lh1[0] == index):
        return default
    m_index = lh1[0]-1
    while 1:
        lh2 = LH(serial,m_index,item)
        if lh2[0] == m_index:
            return default
        if lh2[1] > lh1[1]:
            return lh1
        lh1 = lh2
        m_index = lh1[0] - 1
    return default

def VLs(serial,index,item,level = 1):
    default = []
    m_index = index
    source = serial
    for i in range(level):
        target = []
        while 1:
            ll = LL(source,m_index,item)
            if ll[0] == m_index:
                break
            if ll[1][item] > source[m_index][item]:
                target.insert(0,source[ll[0]])
            m_index =  ll[0]
        
        if len(target) > 0:
            #print('s:{0}----t:{1}'.format(len(source),len(target)))
            source = target
            continue
        else:
            return default
    return target

def VHs(serial,index,item,level = 1):
    default = []
    m_index = index
    source = serial
    for i in range(level):
        target = []
        while 1:
            lh = LH(source,m_index,item)
            #print('{0}\t{1}'.format(m_index,ll[0]))
            if lh[0] == m_index:
                
                break
            if lh[1][item] < source[m_index][item]:
                target.insert(0,source[lh[0]])
            m_index =  lh[0]
        if len(target) > 0:
            #print('s:{0}----t:{1}'.format(len(source),len(target)))
            
            source = target
            continue
        else:
            return default
    return target

def TUL(serial,index,itemh,iteml):
    m_index = index-1
    m_max = serial[index][itemh]
    m_min = serial[index][iteml]
    default = (m_min,m_max,m_index)
    while 1:
        vlv = VLV(serial,m_index,iteml)
        vhv = VHV(serial,m_index,itemh)
        if vlv[0] == m_index:
            return default
        if vhv[0] == m_index:
            return default
        
        if vlv[0] > vhv[0]:
            if vlv[1] < m_max * 0.8:
                return default
            else:
                m_min = min(vlv[1],m_min)
                m_index = vlv[0]
                default = (m_min,m_max,m_index)
        else:
            if vhv[1] > m_min * 1.2:
                return default
            else:
                m_max = max(vhv[1],m_max)
                m_index = vhv[0]
                default = (m_min,m_max,m_index)
    pass


def VDC(serial,index,interval):
    default = None
    if index < interval:
        return default
    vol = 0
    stack = 0.0
    for i in range(index-interval,index+1):
        vol += serial[i][5]
        stack += serial[i][6]
    return (index,stack/vol)

def LTH(serial,index):
    default = None
    for i in range(index-1,0,-1):
        if serial[i][4] > serial[i+1][4]:
            if serial[i][4] >= serial[i-1][4]:
                return serial[i]
    return default

def CMA(serial,index,level=100):
    rtn = 0
    sum_acc = 0.0
    sum_vol = 0
    sum_turn = 0.0
    for i in range(index-1,0,-1):
        sum_turn += float(serial[i][7])
        sum_vol += float(serial[i][6])
        sum_acc += float(serial[i][5])
        if sum_turn > level:
            rtn = sum_acc/sum_vol
            break
    return rtn

def CMI(serial,index,level=100):
    rtn = 0
    sum_acc = 0.0
    sum_vol = 0
    sum_turn = 0.0
    for i in range(index-1,0,-1):
        sum_turn += float(serial[i][7])
        sum_vol += float(serial[i][6])
        sum_acc += float(serial[i][5])
        if sum_turn > level:
            rtn = index-i
            break
    return rtn

def HHV(serial,index,period):
    rtn = serial[indx][2]
    for i in range(index-1,index-period-1,-1):
        if rtn < serial[i][2]:
            rtn = serial[i][2]
    return rtn

def LLV(serial,index,period):
    rtn = serial[indx][3]
    for i in range(index-1,index-period-1,-1):
        if rtn < serial[i][3]:
            rtn = serial[i][3]
    return rtn

class Qualification:
    from abc  import abstractmethod
    def __init__(self,code,type,cqcx = False):
        self.qfile = QualifyFile(code,type,cqcx)
        if cqcx:
            self.dfile = CqcxDailyFile(code)
        else:
            self.dfile = DailyFile(code)
        self.q_data = self.qfile.getData()
        self.d_data = self.dfile.getData()
        
        self.type = type
        self.code = code
        
    def getNextIndex(self):
        if self.q_data == []:
            return 0
        for i in range(0,len(self.d_data)):
            if self.q_data[-1][0] < self.d_data[i][0]:
                return i
        return -1
    
    def calcAppends(self):
        last = self.getNextIndex()
        
        if last == -1:
            return None
        d_data = self.d_data
        self.appends = []
        for i in range(last,len(d_data)):
            
            filter = False
            append = self.calcValue(i)
            self.q_data.append(append)
            for appenditem in append:
                if type(appenditem) == type(None):
                    filter = True
                    continue
            if filter:
                continue
            
            self.appends.append(append)
            
        return self.appends
    
    def writeAppends(self):
        appends = self.calcAppends()
        if appends == None:
            return
        if len(appends) == 0:
            return

        self.qfile.appendData(appends)
        #for append in appends:
            #self.q_data.append(append)
        pass
    
    @abstractmethod
    def calcValue(self,input_index,d_data = None,q_data = None):
        '''计算技术值'''
        pass
    
    def getCurrent(self,cur,mid=False):
        default = None
        if len(self.d_data) == 0:
            return default
        if len(self.q_data) == 0:
            return default
        
        d_data = self.d_data
        q_data = self.q_data
        index = (len(self.d_data)-1)
        if mid:
            for d in self.d_data:
                if d[0] == cur[0]:
                    index = (self.d_data.index(d))
                    d_data = self.d_data[:index]
                    break
            for q in self.q_data:
                if q[0] == cur[0]:
                    q_data = self.q_data[:self.q_data.index(q)]
                    break
        d_data.append(cur)
        rtn = self.calcValue(index,d_data,q_data)
        return rtn
    
    def getValues(self):
        self.q_data = self.qfile.getData()
        return self.q_data
    
    def getValue(self,date):
        for v in self.getValues():
            if v[0] == date:
                return v
        pass

class RNG(Qualification):
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'Rng'+'_'+cycle,cqcx=False)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx=False)
            self.d_data = self.dfile.getData()
        kdjfile = KDJ(code,cycle)
        self.kdjdata = kdjfile.getValues()
        pass

    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        d_date = d_data[index][0]
        d_close = d_data[index][4]
        value = [d_date,d_close]
        kdjindex = getIndexOfSerial(self.kdjdata,d_date)
        if kdjindex == -1:
            return (None,)
        rl0 = None
        rh0 = None
        rl1 = None
        rh1 = None
        findl = True
        findkdjindex0 = kdjindex
        if self.kdjdata[kdjindex][2] < self.kdjdata[kdjindex][3]:
            findl = False
        for i in range(40):
            if findl:
                findkdjindex1 = VLV(self.kdjdata,findkdjindex0,4,3)[0]
            else:
                findkdjindex1 = VHV(self.kdjdata,findkdjindex0,4,3)[0]
            if findkdjindex0 == findkdjindex1:
                return (None,)
            findkdjindex0 = findkdjindex1
            findkdj = self.kdjdata[findkdjindex0]
            finddaily = self.d_data[getIndexOfSerial(self.d_data,findkdj[0])]
            #print(finddaily)
            
            if findl:
                if findkdj[2]>findkdj[3]:
                    continue
                findl = False
                if(d_close > finddaily[4]):#(d_close / 1.05 < finddaily[4]) & 
                    if (rl0 == None):
                        rl0 = finddaily[4]
                    elif (rl1 == None):
                        rl1 = finddaily[4]
            else:
                if findkdj[2]<findkdj[3]:
                    continue
                findl = True
                if(d_close < finddaily[4]):# (d_close * 1.05 > finddaily[4]) & 
                    if (rh0 == None):
                        rh0 = finddaily[4]
                    elif (rh1 == None):
                        rh1 = finddaily[4]
            if (not (rh1 == None)) & (not (rl1 == None)):
                break
        
        if (rh1 == None) | (rl1 == None):
            return (None,)
        
        value.append(rl0)
        value.append(rh0)
        value.append(rl1)
        value.append(rh1)
        
        return tuple(value)    

class DTMA(Qualification):
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'dmt'+'_'+cycle,cqcx=False)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx=False)
            self.d_data = self.dfile.getData()
        mafile = MA(code,cycle)
        self.madata = mafile.getValues()
        pass

    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        d_date = d_data[index][0]
        d_close = d_data[index][4]
        value = [d_date,d_close]
        maindex = getIndexOfSerial(self.madata,d_date)
        if maindex == -1:
            return (None,)
        ma = self.madata[maindex]
        r1 = 1/(1/5)*((1/5*ma[1]))
        r2 = 1/(1/5+1/10)*((1/5*ma[1])+(1/10*ma[2]))
        r3 = 1/(1/5+1/10+1/30)*((1/5*ma[1])+(1/10*ma[2])+(1/30*ma[3]))
        r4 = 1/(1/5+1/10+1/30+1/60)*((1/5*ma[1])+(1/10*ma[2])+(1/30*ma[3])+(1/60*ma[4]))
        
        value.append(r1)
        value.append(r2)
        value.append(r3)
        value.append(r4)
        
        return tuple(value)    

class BLC(Qualification):
    
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'Blc'+'_'+cycle,cqcx=False)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx=False)
            self.d_data = self.dfile.getData()

    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        value = [d_data[index][0],]
        v0 = VLV(d_data,index,5,3)
        if v0[0] == index:
            return (None,)
        v1 = VLV(d_data,v0[0],5,3)
        if v1[0] == v0[0]:
            return (None,)
        value.append(d_data[v1[0]][4])
        mi = min([l for d,o,h,l,c,a,v in d_data[v1[0]:v0[0]+1]])
        ma = max([h for d,o,h,l,c,a,v in d_data[v1[0]:v0[0]+1]])
        value.append(ma)
        value.append(mi)
        value.append(d_data[v0[0]][4])
        vol = 0
        amount = 0.0
        for dta in d_data[v1[0]:v0[0]+1]:
            vol += dta[6]
            amount += dta[5]
        value.append(amount)
        value.append(vol)
        #print(value)
        value.append(v0[0])

        return tuple(value)    

class DLT(Qualification):
    
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'Dlt'+'_'+cycle,cqcx=False)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx=False)
            self.d_data = self.dfile.getData()

    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        level = [5,10,30,60]
        value = [d_data[index][0],]
        for li in range(0,len(level)):
            if 2*level[li] <= index:
                v = 0.0
                for ii in range(0,level[li]):
                    v +=(d_data[index-ii][4] - d_data[index-ii-level[li]][4])/level[li]
                av = v/level[li]
                value.append(av)
            else:
                value.append(None)
        return tuple(value)

class MA(Qualification):
    
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'Ma'+'_'+cycle,cqcx=False)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx=False)
            self.d_data = self.dfile.getData()

    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        level = [5,18,30,60]
        value = [d_data[index][0],]
        for li in range(0,len(level)):
            if level[li] <= index:
                sum = 0.0
                for i in range(index-level[li]+1,index+1):
                    sum = sum + d_data[i][4]
                value.append(round((sum/level[li]),2))
            else:
                value.append(None)
        return tuple(value)

class STMA(Qualification):
    def __init__(self,code,cycle='D'):
        Qualification.__init__(self,code,'STMa',cqcx=False)
        self.cycle = cycle
        self.dfile = StackFile(code)
        self.d_data = self.dfile.getData()
    
    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        level = [5,18,30,60]
        value = [d_data[index][0],]
        for li in range(0,len(level)):
            if level[li] <= index:
                sum = 0.0
                for i in range(index-level[li]+1,index+1):
                    sum = sum + d_data[i][1]
                value.append(round((sum/level[li]),2))
            else:
                value.append(None)
        return tuple(value)

    
class KDJ(Qualification):
    
    def __init__(self,code,cycle = 'D',cqcx = False):
        Qualification.__init__(self,code,'KDJ'+'_'+cycle,cqcx)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx)
            self.d_data = self.dfile.getData()
        if cycle == 'H':
            self.dfile = HourFile(code)
            self.d_data = self.dfile.getData()
        if cycle == 'Q':
            self.dfile = QuarterFile(code)
            self.d_data = self.dfile.getData()

        
    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        if self.cycle == 'W':
            temp_d = []
            temp_d.append(d_data[index])
            for i in range(index-1,-1,-1):
                if d_data[0][0].strftime('%W') != d_data[i][0].strftime('%W'):
                    temp_d.insert(0,d_data[i])
                    if len(temp_d) > 81:
                        break
        else:
            temp_d = []
            for i in range(index,-1,-1):
                temp_d.insert(0,d_data[i])
                if len(temp_d)>81:
                    break
        #q_data = q_data + self.appends
        if len(temp_d) < 81:
            return (d_data[-1][0],None,None,None,None)

        hhv = temp_d[-1][2]
        llv = temp_d[-1][3]
        for i in range(len(temp_d)-81,len(temp_d)):
            if temp_d[i][2]>hhv:
                hhv = temp_d[i][2]
            if temp_d[i][3]<llv:
                llv = temp_d[i][3]
        
        if hhv == llv:
            return (d_data[-1][0],None,None,None,None)
        
        rsv = 100.0*(d_data[-1][4]-llv)/(hhv-llv)

        pre_k = 0
        pre_d = 0
        if self.cycle == 'W':
            for i in range(len(q_data)-1,-1,-1):
                if q_data[i][0].strftime('%W') != d_data[-1][0].strftime('%W'):
                    pre_k = q_data[i][2]
                    pre_d = q_data[i][3]
                    break
            
        elif self.cycle == 'M':
            for i in range(len(q_data)-1,-1,-1):
                if q_data[i][0].strftime('%M') != d_data[-1][0].strftime('%M'):
                    pre_k = q_data[i][2]
                    pre_d = q_data[i][3]
                    break
        else:
            if len(q_data) > 0:
                pre_rsv = q_data[-1][1]
                pre_k = q_data[-1][2]
                pre_d = q_data[-1][3]
        
        k = SMA(pre_k,rsv,18,1)
        
        d = SMA(pre_d,k,18,1)
        j = 3*k-2*d
        #print((d_data[-1][0],rsv,k,d,j))
        return (d_data[-1][0],rsv,k,d,j)

class MACD(Qualification):
    
    def __init__(self,code,cycle = 'D',cqcx = False):
        Qualification.__init__(self,code,'MACD'+'_'+cycle,cqcx)
        self.cycle = cycle
        if cycle == 'W':
            self.dfile = WeeklyFile(code,cqcx)
            self.d_data = self.dfile.getData()

        
    def calcValue(self,index,d_data=None,q_data=None):
        #q_data = self.q_data + self.appends
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        c = d_data[index][4]

        pre_ema12 = 0
        pre_ema26 = 0
        pre_dea = 0
        if self.cycle == 'W':
            for i in range(len(q_data)-1,-1,-1):
                if q_data[i][0].strftime('%W') != d_data[index][0].strftime('%W'):
                    pre_ema12 = q_data[i][1]
                    pre_ema26 = q_data[i][2]
                    pre_dif = q_data[i][3]
                    break
            
        elif self.cycle == 'M':
            for i in range(len(q_data)-1,-1,-1):
                if q_data[i][0].strftime('%M') != d_data[index][0].strftime('%M'):
                    pre_ema12 = q_data[i][1]
                    pre_ema26 = q_data[i][2]
                    pre_dea = q_data[i][4]
                    break
        else:
            if len(q_data) > 1:
                pre_ema12 = q_data[-1][1]
                pre_ema26 = q_data[-1][2]
                pre_dif = q_data[-1][3]
                pre_dea = q_data[-1][4]
        ema12 = EMA(pre_ema12,c,12)
        ema26 = EMA(pre_ema26,c,26)
        dif = ema12 - ema26
        #print(pre_dea)
        dea = EMA(pre_dea,dif,9)
        
        macd = (dif-dea)*2.0
        #print((d_data[-1][0],rsv,k,d,j))
        return (d_data[index][0],ema12,ema26,dif,dea,macd)
    
class LVL(Qualification):
    def __init__(self,code,cqcx = False):
        Qualification.__init__(self,code,'LVL',cqcx)
        
    def calcValue(self,index,d_data=None,q_data=None):
        if d_data == None:
            d_data = self.d_data
        if q_data == None:
            q_data = self.q_data
        if index < 10:
            return (d_data[index][0],None,None,None)
            
        kt1 = self.getKValue(index)
        if kt1 != None:
            kt2 = self.getKValue(kt1[3])
            if kt2 != None:
                kv = self.checkKValue(kt2,index)
                if kv != None:
                    #data
                    return (d_data[index][0],kv,index-kt2[3],kt2[1])
        return (d_data[index][0],None,None,None)
        
    def getKValue(self,index):
        k = 0
        rtn = None
        for ix in range(index-1,-1,-1):
            k1 = (self.d_data[index][3]-self.d_data[ix][3])/(index-ix)/self.d_data[ix][3]
            if(k1>=k):
                k = k1
                rtn = (index,k,self.d_data[index][3],ix,self.d_data[ix][3])
        return rtn
        
    def checkKValue(self,kv,index):
        if(len(kv)==0):
            return None
        return (kv[1] * kv[4] * (index-kv[3])+kv[4])
        
        
class HVH(Qualification):
    def __init__(self,code,cqcx = False):
        Qualification.__init__(self,code,'HVH',cqcx)
    
    def calcValue(self,index):
        if index < 10:
            return (self.d_data[index][0],None,None,None)
            
        kt1 = self.getKValue(index)
        if kt1 != None:
            kt2 = self.getKValue(kt1[3])
            if kt2 != None:
                kv = self.checkKValue(kt2,index)
                if kv != None:
                    return (self.d_data[index][0],kv,index-kt2[3],kt2[1])
        return (self.d_data[index][0],None,None,None)
        
    def getKValue(self,index):
        k = 0
        rtn = None
        for ix in range(index-1,-1,-1):
            k1 = (self.d_data[index][2]-self.d_data[ix][2])/(index-ix)/self.d_data[ix][2]
            if(k1<=k):
                k = k1
                rtn = (index,k,self.d_data[index][2],ix,self.d_data[ix][2])
        return rtn
    
    def checkKValue(self,kv,index):
        if(len(kv)==0):
            return None
        return (kv[1] * kv[4] * (index-kv[3])+kv[4])

class MVL(Qualification):
    def __init__(self,code,cqcx = False):
        Qualification.__init__(self,code,'MVL',cqcx)
    
    def calcValue(self,index):
        default = (self.d_data[index][0],None,None)
        if index < 1:
            return default
        if index > len(self.d_data)-4:
            return default
        if self.d_data[index][5] > self.d_data[index-1][5]*1.2:
            #if ((self.d_data[index][4] > self.d_data[index-1][4]) & (self.d_data[index+1][4] > self.d_data[index-1][4]) & (self.d_data[index+2][4] > self.d_data[index-1][4]) & (self.d_data[index+3][4] > self.d_data[index-1][4])):
            return (self.d_data[index][0],self.d_data[index-1][4],self.d_data[index][4],self.d_data[index][5])
            #return default
        else:
            return default
        